// Various macros, like SET/CLEAR/TEST_BIT, TRUE/FALSE, etc.
#include "macro.h"

// Chip configurations, like cpu frequency (F_CPU) and desired uart baud rate.
#include "config.h"

// LCD driver, use 'lcd'-variable as FILE-pointer to std. IO-functions.
#include "lcd_driver.h"

// AVR include files, contains PORTx,PINx and other register defines, plus
// SIGNAL() and INTERRUPT() macros, etc.
#include <avr/io.h>
#include <avr/interrupt.h>

#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include "uart.h"
#include "motor.h"

#include "lcd_driver.h"




void init(void)
{
	//change clock to 8MHz

	CLKPR = (1<<CLKPCE);
	CLKPR = 0;


	// The butterfly bootloader has the "undocumented feature" of disabling
	// digital input on PORTE(2..3) and PORTF, which would block certain
	// features such as left/right of the butterfly joystick. In order to
	// fix this we must clear the Digital Input Disable Registers.

	DIDR0 = 0x00;
	DIDR1 = 0x00;

	// Initialize all submodules
	cli();
	lcd_init();
	// Set default standard-out to be LCD-display.
	stdout = lcd;

	uartinit();
	// Disable all external interrupts. (Otherwise some pin-change interrupts
	// etc. may still be enabled, causing the processor to spontaneously reset).
	EIMSK = 0x00;

	// Enable interrupts
	sei();

	return;
}


int main(void)
{
	init();

	int speed = 0;
	unsigned char pushed_long = 0;

	//set joystick as input with pullup
	DDRB &= ~(1<<PB4)|(1<<PB6)|(1<<PB7);
	PORTB |= (1<<PB4)|(1<<PB6)|(1<<PB7);

	printf("start\n");
	MotorStart();
	_DELAY_MS(1000);
	MotorStartWithError();

	printf("ok\n");
	_DELAY_MS(1000);
	MotorStartMotor();
	_DELAY_MS(1000);
	MotorStart();
	while (1){

		
		
		if(!(PINB&(1<<PB6))){
			pushed_long++;
			if(pushed_long>5){
				pushed_long = 6;
				speed+=10;
			}else{
				speed++;
			}

			if(speed>500){
				speed = 500;
			}

			MotorSetSpeedRef(speed);		
		}
		else if(!(PINB&(1<<PB7))){
			pushed_long++;
			if(pushed_long>5){
				pushed_long = 6;
				speed-=10;
			}else{
				speed--;
			}
			if(speed<0){
				speed = 0;
			}

			MotorSetSpeedRef(speed);
		}
		else{
			pushed_long = 0;
		}

		if((!(PINB&(1<<PB4)))){
			printf("off\n");
			_DELAY_MS(500);
			wdt_enable(WDTO_15MS);
			while(1); //wait for watchdog
		}

		printf("%d\n",speed);

		_DELAY_MS(200);



	}
}

