

#include "motor.h"
#include <inttypes.h>
#include <util/crc16.h>

void MotorSend(int len, char data[]){
	uint8_t pos = 0; //Posisjon i utdata-tabellen
	
	char utdata[100];

	/*Pakkene er p formen:
	/
	/	0xFF | N | Header Checksum | data1 | ... | dataN | Data Checksum |
	/
	/ 	0xFF markerer start p pakke. Andre steder 0xFF inntreffer, blir
	/ 	den kodet som to pflgende 0xFF.
   */


	//Header byte
	utdata[pos]=0xFF; 
	pos++;

	//N - number of decoded data bytes
	utdata[pos]=len;
	pos++;

	//Header checksum
	char check1 = ~(0xFF + (char)len) + 1;
	utdata[pos]= check1;
	pos++;
	if(check1 == 0xFF){
		utdata[pos] = 0xFF;
		pos++;
	}

	//Data
	int t;
	uint16_t crc = 0;
	for(t=0;t<len;t++){
		utdata[pos] = data[t];
		crc = _crc16_update(crc,utdata[pos]);
		pos++;
		if(data[t]==0xFF){
			utdata[pos]=0xFF;
			pos++;
		}
	}

	//Data checksum
	char check2 = (char)(crc>>8)&0xFF;
	char check3 = (char) crc&0xFF;

	utdata[pos] = check2;
	pos++;
	if(check2 == 0xFF){
		utdata[pos] = 0xFF;
		pos++;
	}
	utdata[pos] = check3;
	pos++;
	if(check3 == 0xFF){
		utdata[pos] = 0xFF;
		pos++;
	}

	//Sender data
	for(t=0;t<pos;t++){
		SendByte(utdata[t]);
	}
}

void MotorPing(){
	char msg[2] = {0x01,0x00};
	MotorSend(2,msg);
}

void MotorStart(){
	char msg[2] = {0x02,0x00};
	MotorSend(2,msg);
}

void MotorStartWithError(){
	char msg[5] = {0x10,0x01,0x55,0x04,0x00};
	MotorSend(5,msg);
}

void MotorStartMotor(){
	char msg[5] = {0x10,0x01,0x55,0x00,0x00};
	MotorSend(5,msg);
}

void MotorStop(){
	char msg[2] = {0x04,0x00};
	MotorSend(2,msg);
}

void MotorReset(){
	char msg[2] = {0x05,0x00};
	MotorSend(2,msg);
}

void MotorSetSpeedRef(int16_t speed_rpm){
	//Filling the message. Leaves the speed referance empty
	char msg[10] = {0x10,0x02,0x3F,0x04,0x00,0x00,0x00,0x00,0x00,0x00};

	//F7$10$02$3F$04$00$00$00$00$10$00
	
	//Converting RPM value into Q.20 PU value.
	int32_t speed_q20= ((int32_t)speed_rpm)*5958; //5958=2^20/176
	
	//Filling in the speed referance
	int t=0;
	for(t=0;t<4;t++){
		msg[9-t]= (char)(speed_q20>>(8*(3-t)))&0xFF;
	}

	//Sending the message
	MotorSend(10,msg);
}

int32_t MotorGetAksOL(){
	return 456;
}

int32_t MotorSetAksOL(int noe){
	char msg[10] = {0x10,0x02,0x3F,0x04,0x27,0x00,0x00,0x00,0x00,0x00};
	return 456;
}	

void MotorGetSpeedRef(){
	//IKKE FERDIG
	char msg[2] = {0x01,0x00};
	MotorSend(2,msg);
}
void MotorGetSpeed(){
	//IKKE FERDIG
	char msg[6] = {0x10,0x03,0x56,0x02,0x01,0x00};
	MotorSend(6,msg);
}


void SkrivMotorError(uint8_t ec){
	
}
