/* rev 1.0 01/2010 				written by Anders Guldahl			     */

#include <stdio.h>
#include <string.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

#include "macro.h"
#include "uart_debug.h"
#include "status_led.h"
#include "voltage_adc.h"
#include "can.h"
#include "SPI_routines.h"
#include "main.h"
#include "mcp23S17.h"
#include "led_driver.h"
#include "uart_motor.h"
#include "motor.h"
#include "../common_files/eeprom.h"
#include "../common_files/status_structs.h"

BOARD_STATUS board_status;
SIGNALS_LIGHTS signals_lights;


//initialize module
unsigned char init(void){

	board_status.reset_source = MCUSR;
	MCUSR = 0;
	wdt_disable();

	eepromResetBootByte();
	add_to_PowerOnCount();

	USART_Init_debug();
	USART_Init();
	init_can();
	init_adc();
	spi_init();
	init_status_led();
	init_io_expander();
	init_led_driver();

		

	heart_beat(1);
	set_status_led(GREEN);


	sei();
	return 0;
}





int main(){


	init();


	set_status_led(GREEN);


	char output_string[200];
	unsigned char data, brakelight_on;
	brakelight_on = 0;
	int counter = 0;
	int test_recv = -1;
	unsigned char battery_light = 0xFE;
	
	unsigned char motor_start_state = 0;
	unsigned char motor_stopp_state = 0;
	unsigned char motor_stopped = 1;
	unsigned int motorstart_systick = 0; 

	unsigned char buffer[30];
	unsigned int last_system_tick = 0;
	unsigned char motor_speed_ref = 0;
	unsigned char new_motor_command = 0;
	
	union can_frame send;
	union can_frame msg;
	union can_frame *recv;
	recv = &msg;


	printf("Times powered on: %d reset_source %x\n\r", board_status.times_powered_on, board_status.reset_source);

	while(1){

		counter ++;

		board_status.mcu_voltage = read_mcu_voltage();
		board_status.board_voltage = read_board_voltage();
		board_status.input_voltage = read_input_voltage();
		
		

		sprintf(output_string,"counter: %d mcu %d,board %d,input %d\r\n",counter,board_status.mcu_voltage,board_status.board_voltage,board_status.input_voltage);
		//printf(output_string);

		/*
		if(last_system_tick <= get_system_ticks()-6){

			MotorGetVoltage();


			last_system_tick = get_system_ticks();
		}
		*/
		

		if(motor_start_state == 1){
			
			MotorStart();
			_DELAY_MS(50);
			MotorStartWithError();
			motor_start_state = 0;
			_DELAY_MS(50);
			MotorStartMotor();
			_DELAY_MS(50);
			MotorStart();
			motor_stopped = 0;
				
		}


		if(motor_stopp_state){
			motor_stopped = 1;
			MotorStop();
			motor_start_state = 0;
			motor_stopp_state = 0;
			_DELAY_MS(50);
			MotorReset();
		}


	

		//quick hack uart-debug commands, mostly changing LED-color, useful for testpurposes

		data = USART_Receive_debug(NULL);

		switch (data){
		case'1':
			heart_beat(1);
		break;
		case'2':
			slow_breathing(1);
		break;
		case'3':
			fast_breathing(1);
		break;
		case'4':
			set_status_led(RED);
		break;
		case'5':
			set_status_led(GREEN);
		break;
		case'6':
			set_status_led(BLUE);
		break;
		case'7':
			set_status_led(YELLOW);
		break;
		case'8':
			while(1);
		break;
		case'9':

		break;
		case'q':

		break;
		case'w':
			set_status_led(LED_OFF);
		break;
		case't':
			

		break;
		case'r':
			wdt_enable(WDTO_15MS);
			while(1); //wait for watchdog

		break;
	
		}





		//send and receive CAN-messages

	
		while(new_can_frame()){//receive all new can-frames
			recv = read_can_frame();
			next_can_frame();
	
			if(recv->id == 0x6C1){
				OCR2A = recv->data[2]; 
				OCR3AL = recv->data[3];
				signals_lights.backlight_brightness = recv->data[0];			
				signals_lights.brakelight_brightness = recv->data[1];

				if(brakelight_on){
					OCR3BL = recv->data[1];
					OCR3CL = recv->data[1];
					
				}else{
					OCR3BL = 0;				
					OCR3CL = recv->data[0];
				}
			
			}
			if(recv->id == 0x6C0){
				brakelight_on = recv->data[0];
				if(brakelight_on){
					motor_stopp_state = 1;
					OCR3BL = signals_lights.brakelight_brightness;
					OCR3CL = signals_lights.brakelight_brightness;
					
				}else{
					OCR3BL = 0;				
					OCR3CL = signals_lights.backlight_brightness;
					if(motor_stopped){
						motor_start_state = 1;

					}
					
					
				}
			}
			if(recv->id == 0x6C2){
				if(recv->data[0]){
					
					motor_start_state = 1;
					
				}
			}
			if(recv->id == 0x6C3){
				motor_speed_ref = recv->data[0];
				MotorSetSpeedRef((int)motor_speed_ref*6);
					
			}
			if(recv->id == 0x6C4){
				
				//turn on/off safetyloop (load relay/h2 valve)
				if(recv->data[0] == 1){
				
					set_io_bit('A',7);
				}
				else if(recv->data[0] == 0){
				
					clear_io_bit('A',7);
				}
			
			}
			if(recv->id == 0x6D0){
				
				//batteryLED		
				int battery_voltage = (recv->data[0]<<8) | recv->data[1];
				ShowBatteryLED(battery_voltage);
			
			}
		}


		//send msg over CAN
 		SendStatusMsg(0x0C);
	
		_DELAY_MS(10);
	}
}

void SendStatusMsg(unsigned char status){
		
	static unsigned int last_system_ticks = 0;
	if(last_system_ticks != get_system_ticks()){
		union can_frame send;
		send.id = 0x7D0;			
		send.length = 1;
		send.rbr = 0;
		send.rb0 = 0;
		send.data[0] = status;
		send_can_frame(&send);

		//status to dashboard
		send.id = 0x758;
		send.length = 8;
		send.rbr = 0;
		send.rb0 = 0;
		send.data[0] = status;
		int i;
		for(i=1;i<8;i++){
			send.data[i] = 0;
		}
		send_can_frame(&send);		

		last_system_ticks = get_system_ticks();
	}
	
}

