
/* rev 1.0 02/2010 				written by Anders Guldahl			     */

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "macro.h"
#include "SPI_routines.h"
#include "mcp23S17.h"



unsigned char init_io_expander(void){

	
	SD_CS_ASSERT;

	SPI_transmit(0x40);	//write mode
	SPI_transmit(0x00); //start at register 0x00

	SPI_transmit(0x07); //reg:0x00 Port A 0,1,2 input, rest output
	SPI_transmit(0x00); //reg:0x01 output port B
	SPI_transmit(0x00); //reg:0x02 polarityA
	SPI_transmit(0x00); //reg:0x03 polarityB
	SPI_transmit(0x00); //reg:0x07 INTA could be enabled on pin 0,1 and 2 for switches inside enclosure, but disabled now
	SPI_transmit(0x00); //reg:0x05 INT B
	SPI_transmit(0x00); //reg:0x06 DEFVALA
	SPI_transmit(0x00); //reg:0x07 DEFVALB
	SPI_transmit(0x00); //reg:0x08 INTCONA //interrupt on pin-change is active 0
	SPI_transmit(0x00); //reg:0x09 INTCONB
	SPI_transmit(0x04); //reg:0x0A IOCON	
	SPI_transmit(0x04); //reg:0x0B IOCON	//same as above?
	SPI_transmit(0x00); //reg:0x0C PULLUPA
	SPI_transmit(0x00); //reg:0x0D PULLUPB
	SPI_transmit(0x00); //reg:0x0E INTFLAG A
	SPI_transmit(0x00); //reg:0x0F INTFLAG B
	SPI_transmit(0x00); //reg:0x10 INT CAPTURE A
	SPI_transmit(0x00); //reg:0x11 INT CAPTURE B
	SPI_transmit(0x00); //reg:0x12 GPIOA
	SPI_transmit(0x00); //reg:0x13 GPIOB
	SPI_transmit(0x00); //reg:0x14 OUTPUT LATCH A
	SPI_transmit(0x00); //reg:0x15 OUTPUT LATCH B
	
		
	SD_CS_DEASSERT;


	//init io_A interruptpin on avr pin int1, enable pull-up, trigger on falling edge
	
	DDRD &= ~(1<<PD1); //set as input
	PORTD |= (1<<PD1); //enable pullup

	//interrupt disabled, buttons not needed
	//EICRA &= ~((1<<ISC11)|(1<<ISC10)); //low level interrupt
	
	// ENABLE_EXT_INT1; //enable interrupt on pin1




	return 0;
}


unsigned char set_io_bit(char port, unsigned char pin){
	
	DISABLE_EXT_INT1;
	if(port == 'A'){
		SD_CS_ASSERT;
		SPI_transmit(0x41);	//read mode
		SPI_transmit(0x14); //register 0x14 OLATA
		unsigned char port = SPI_receive();
		SD_CS_DEASSERT;

		port |= (1<<pin); // set bit
		
		//write back
		SD_CS_ASSERT;
		SPI_transmit(0x40);	//write mode
		SPI_transmit(0x14); //register 0x14 OLATA
		SPI_transmit(port);
		SD_CS_DEASSERT;

	
	}
	else if(port == 'B'){
		SD_CS_ASSERT;
		SPI_transmit(0x41);	//read mode
		SPI_transmit(0x15); //register 0x15 OLATB
		unsigned char port = SPI_receive();
		SD_CS_DEASSERT;

		port |= (1<<pin); // set bit
		
		//write back
		SD_CS_ASSERT;
		SPI_transmit(0x40);	//write mode
		SPI_transmit(0x15); //register 0x15 OLATB
		SPI_transmit(port);
		SD_CS_DEASSERT;


	}
	ENABLE_EXT_INT1;
	return 0;
}

unsigned char clear_io_bit(char port, unsigned char pin){
	DISABLE_EXT_INT1;
	if(port == 'A'){
		SD_CS_ASSERT;
		SPI_transmit(0x41);	//read mode
		SPI_transmit(0x14); //register 0x14 OLATA
		unsigned char port = SPI_receive();
		SD_CS_DEASSERT;

		port &= ~(1<<pin); // clear bit
		
		//write back
		SD_CS_ASSERT;
		SPI_transmit(0x40);	//write mode
		SPI_transmit(0x14); //register 0x14 OLATA
		SPI_transmit(port);
		SD_CS_DEASSERT;

	
	}
	else if(port == 'B'){
		SD_CS_ASSERT;
		SPI_transmit(0x41);	//read mode
		SPI_transmit(0x15); //register 0x15 OLATB
		unsigned char port = SPI_receive();
		SD_CS_DEASSERT;

		port &= ~(1<<pin); // clear bit
		
		//write back
		SD_CS_ASSERT;
		SPI_transmit(0x40);	//write mode
		SPI_transmit(0x15); //register 0x15 OLATB
		SPI_transmit(port);
		SD_CS_DEASSERT;


	}
	ENABLE_EXT_INT1;
	return 0;
}

void write_led_gpio_byte(unsigned char byte){

	DISABLE_EXT_INT1;
	SD_CS_ASSERT;
	SPI_transmit(0x40);	//write mode
	SPI_transmit(0x15); //register 0x15 OLATB
	SPI_transmit(byte);
	SD_CS_DEASSERT;
	ENABLE_EXT_INT1;
}




ISR(INT1_vect){
	
	
	
	SD_CS_ASSERT;
	
	//read INTCAPA
	SPI_transmit(0x41);	//read mode
	SPI_transmit(0x10); //register 0x10 INTCAPA
	unsigned char io_reg = SPI_receive();
	SD_CS_DEASSERT;
	
	

}

