

#include <stdio.h>
#include <inttypes.h>
#include <util/crc16.h>
#include "motor.h"
#include "uart_motor.h"
#include "integer.h"
#include "uart_debug.h"
#include "macro.h"






int PECCP50Decode(PECCP50 *p, int data){

	if (data < 0 || data > 0xFF) {
		/* Resynch on <error> */
		p->state = PECCP50_SYNCH;
		return -2;
	}
	switch (p->state) {
		case PECCP50_VALID:
			p->state = PECCP50_SYNCH;
			/* fallthrough */
		case PECCP50_SYNCH:
			if (data == 0xFF){
				p->state = PECCP50_LEN;
			}
			else{
				return -1;
			}
		break;
		case PECCP50_LEN:
			if (data > 0 && data < 0xFF) {
				/* Valid length */
				p->len = data;
				p->lastff = FALSE;
				p->state = PECCP50_LSUM;
			} else if (data == 0xFF) {
				/* FF FF - last FF might be a synch, continue in
				LEN state */
				return -1;
			} else {
				/* FF 00 - resynch */
				p->state = PECCP50_SYNCH;
				return -2;
			}
		break;
		case PECCP50_LSUM:
		case PECCP50_DATA:
		case PECCP50_ESUM:
			if (p->lastff) {
				if (data > 0 && data < 0xFF) {
					/* A synch (!) */
					p->len = data;
					p->state = PECCP50_LSUM;
					return -1;
				} else if (data == 0xFF) { /* FF FF -> FF */
					p->lastff = FALSE;
				} else {/* FF 00 - resynch */
					p->state = PECCP50_SYNCH;
					return -2;
				}
			} else if (data == 0xFF){
				p->lastff = TRUE;
			}
			if (!p->lastff) {
				switch (p->state) {			
					case PECCP50_LSUM:
						if (((0xFF + p->len + data) & 0xFF) != 0) {
							/* Checksum error -> resynch */
							p->state = PECCP50_SYNCH;
							return -3;
						} else {
							p->iter = &p->buf[0];
							p->pos = 0;
							p->sum = 0;
							p->state = PECCP50_DATA;
						}
					break;
					case PECCP50_DATA:
						*p->iter++ = data;
						p->sum += data;
						if (++p->pos == p->len){
							p->state = PECCP50_ESUM;
						}
					break;
					case PECCP50_ESUM:
						if (((p->sum + data) & 0xFF) == 0) {
							/* Packet done ! */
							p->state = PECCP50_VALID;
							return p->len; /* ...and there were much rejoicing */
						}
						/* Checksum error -> resynch */
						p->state = PECCP50_SYNCH;
						return -4;
					default:
						p->state = PECCP50_SYNCH;
				}
			}
		break;
		default:
			p->state = PECCP50_SYNCH;
	}

	return 0;
}


void MotorSend(int len, char data[]){
	uint8_t pos = 0; //Posisjon i utdata-tabellen
	
	char utdata[100];

	/*Pakkene er p formen:
	/
	/	0xFF | N | Header Checksum | data1 | ... | dataN | Data Checksum |
	/
	/ 	0xFF markerer start p pakke. Andre steder 0xFF inntreffer, blir
	/ 	den kodet som to pflgende 0xFF.
   */


	//Header byte
	utdata[pos]=0xFF; 
	pos++;

	//N - number of decoded data bytes
	utdata[pos]=len;
	pos++;

	//Header checksum
	char check1 = ~(0xFF + (char)len) + 1;
	utdata[pos]= check1;
	pos++;
	if(check1 == 0xFF){
		utdata[pos] = 0xFF;
		pos++;
	}

	//Data
	int t;
	uint16_t crc = 0;
	for(t=0;t<len;t++){
		utdata[pos] = data[t];
		crc = _crc16_update(crc,utdata[pos]);
		pos++;
		if(data[t]==0xFF){
			utdata[pos]=0xFF;
			pos++;
		}
	}

	//Data checksum
	char check2 = (char)(crc>>8)&0xFF;
	char check3 = (char) crc&0xFF;

	utdata[pos] = check2;
	pos++;
	if(check2 == 0xFF){
		utdata[pos] = 0xFF;
		pos++;
	}
	utdata[pos] = check3;
	pos++;
	if(check3 == 0xFF){
		utdata[pos] = 0xFF;
		pos++;
	}

	//Sender data
	for(t=0;t<pos;t++){
		uputchar(utdata[t]);
	}
}

void MotorPing(void){
	char msg[2] = {0x01,0x00};
	MotorSend(2,msg);
}

void MotorStart(void){
	char msg[2] = {0x02,0x00};
	MotorSend(2,msg);
}

void MotorStartWithError(void){
	char msg[5] = {0x10,0x01,0x55,0x04,0x00};
	MotorSend(5,msg);
}

void MotorStartMotor(void){
	char msg[5] = {0x10,0x01,0x55,0x00,0x00};
	MotorSend(5,msg);
}

void MotorStop(void){
	char msg[2] = {0x04,0x00};
	MotorSend(2,msg);
}

void MotorReset(void){
	char msg[2] = {0x05,0x00};
	MotorSend(2,msg);
}

void MotorSetSpeedRef(int16_t speed_rpm){
	//Filling the message. Leaves the speed referance empty
	char msg[10] = {0x10,0x02,0x3F,0x04,0x00,0x00,0x00,0x00,0x00,0x00};

	//F7$10$02$3F$04$00$00$00$00$10$00
	
	//Converting RPM value into Q.20 PU value.
	int32_t speed_q20= ((int32_t)speed_rpm)*5958; //5958=2^20/176
	
	//Filling in the speed referance
	int t=0;
	for(t=0;t<4;t++){
		msg[9-t]= (char)(speed_q20>>(8*(3-t)))&0xFF;
	}

	//Sending the message
	MotorSend(10,msg);
}

int32_t MotorGetAksOL(void){
	return 456;
}

int32_t MotorSetAksOL(int noe){
	char msg[10] = {0x10,0x02,0x3F,0x04,0x27,0x00,0x00,0x00,0x00,0x00};
	return 456;
}	

void MotorGetSpeedRef(void){
	//IKKE FERDIG
	char msg[2] = {0x01,0x00};
	MotorSend(2,msg);
}
void MotorGetSpeed(void){
	//IKKE FERDIG
	char msg[6] = {0x10,0x03,0x56,0x02,0x01,0x00};
	MotorSend(6,msg);
}

int MotorGetVoltage(void){

	int data = 0;
	int return_code = 0;
	PECCP50 peccp50;
	peccp50.state = PECCP50_SYNCH;

	MotorPing();
		/*
	char msg[6] = {0x10,0x03,0x56,0x02,0x05,0x00};
	MotorSend(6,msg);
	*/
	_DELAY_MS(100);
	unsigned char buffer[30];
	
	int len = ugets(buffer, 25);
	int i;
	for(i = 0; i<len;i++){
		printf("%x,",buffer[i]);
	
	}
	printf("\n\r");
	/*
	while(return_code == 0){
		data = uart_getchar();
		if(data >= 0 && data <= 0xff){
			return_code = PECCP50Decode(&peccp50, data);		
		}
	
	}
		*/
	return return_code;
	

}

void SkrivMotorError(uint8_t ec){
	
}
