/* rev 1.0 01/2010 				written by Anders Guldahl			     */

#include <stdio.h>
#include <string.h>
#include <inttypes.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

#include "macro.h"
#include "uart_debug.h"
#include "uart_lcd.h"
#include "status_led.h"
#include "voltage_adc.h"
#include "can.h"
#include "SPI_routines.h"
#include "mcp23S17.h"
#include "main.h"
#include "menu.h"
#include "../common_files/status_structs.h"
#include "CANToBluetooth.h"
#include "buzzer.h"
#include "sensirion.h"
#include "timer0.h"
#include "cruise.h"
#include "../common_files/eeprom.h"


// extern structs

BOARD_STATUS board_status;
SIGNALS_LIGHTS signals_lights;
MODULE_STATUS module_status;
RTC_CURRENT rtc_struct;
FUELCELL fuelcell;
CRUISE_CONTROL cruise_control;

//initialize module
unsigned char init(void){

	board_status.reset_source = MCUSR;
	MCUSR = 0;
	wdt_disable();

	eepromResetBootByte();
	add_to_PowerOnCount();

	Timer0Init();
	USART_Init_debug();
	USART_Init_lcd();

	init_can();
	init_adc();
	spi_init();
	init_status_led();
	
	init_io_expander();
	InitBuzzer();
	InitSensirion();


	heart_beat(1);
	set_status_led(GREEN);

	
	

	//default values zero
	signals_lights.left_indicator_light_active  = 0;
	signals_lights.left_indicator_light_on = 0;
	signals_lights.right_indicator_light_active = 0;
	signals_lights.right_indicator_light_on = 0;
	signals_lights.left_indicator_light_brightness = 0;
	signals_lights.right_indicator_light_brightness = 0;

	signals_lights.ticker_period = 0;	

	signals_lights.horn_on = 0;
	signals_lights.low_lights_on = 0;
	signals_lights.low_lights_brightness = 0;
	signals_lights.high_lights_on = 0;
	signals_lights.high_lights_brightness = 0;
	signals_lights.emergency = 0;
	signals_lights.IndicatorTickerOn = 0;


	signals_lights.backlight_brightness = 0;
	signals_lights.brakelight_brightness = 0;

	signals_lights.pedal_position = 0;

	signals_lights.values_changed = 0;

	signals_lights.green_one = 0;
	signals_lights.green_two = 0;
	signals_lights.black_one = 0;

	signals_lights.purge_fuelcell = 0;

	signals_lights.showMode = 0;

	fuelcell.fan_dutycycle = 0;
	
	fuelcell.undervoltage = 0;
	fuelcell.fuelcell_startup = 0;

	cruise_control.set_speed = 0;

	MenuSetDefaultValues();
	sei();
	return 0;
}




int main(){


	init();


	set_status_led(GREEN);


	char output_string[200];
	char *lcd_string = "hello this is a test\n\r";
	unsigned char data;
	int counter = 0;

	union can_frame send;
	union can_frame msg;
	union can_frame *recv;
	recv = &msg;

	int i,j;
	unsigned char state = 100;
	unsigned char gray_pedal_position = 0;
	unsigned char io_registers[22] = {0};
	unsigned char lcd_ready = 0;
	unsigned char lcd_brightness = 0;

	

	printf("Times powered on: %d reset_source %x\n\r", board_status.times_powered_on, board_status.reset_source);

	int input = 0;
	int encoder_rotations = 0;
	unsigned char enc_push = 0;
	unsigned char indicator_light_state = 0;
	unsigned char panel_buttons = 0;
	unsigned char wheel_buttons = 0;

	unsigned char panel_buttons_Bluetooth = 0;
	unsigned char wheel_buttons_Bluetooth = 0;

	cruise_control.set_speed = 0;
	cruise_control.cruise_active = 0;

	for(;;){

		counter ++;

		board_status.mcu_voltage = read_mcu_voltage();
		board_status.board_voltage = read_board_voltage();
		board_status.input_voltage = read_input_voltage();


		


		//sprintf(output_string,"%d time: %d",board_status.input_voltage, get_system_ticks());



		//handle inputs and encoder rotations
		DISABLE_EXT_INTERRUPT;
		encoder_rotations = get_encoder_rotations();
		reset_encoder_rotations();
		enc_push = get_enc_push();
		reset_enc_push();
		panel_buttons = getPanelButtons();
		wheel_buttons = getWheelButtons();
		resetPanelButtons();
		resetWheelButtons();

		ENABLE_EXT_INTERRUPT;

		panel_buttons |= panel_buttons_Bluetooth;	
		wheel_buttons |= wheel_buttons_Bluetooth;
		panel_buttons_Bluetooth = 0;
		wheel_buttons_Bluetooth = 0;

		//handle lcd if lcd is ready after small wait on startup
		if(!lcd_ready){
			if(get_system_ticks() >= 3){
				lcd_ready=1;
				init_spark_lcd();
			}
		}
		else{

			LcdScreenStateMachine(encoder_rotations, enc_push);
			
			enc_push = 0;
			encoder_rotations = 0;		
		}
		
	
		//send light-command over CAN-BUS
		sendCANmsgLights();
						
		//reset status_struct for modules, do not reset watchdog
		ResetModuleStatus(0);


		//handle button pushes
		HandleButtonPushes(panel_buttons, wheel_buttons);
		panel_buttons = 0;
		wheel_buttons = 0;

		//handle LED's in buttons
		HandleButtonLED();

		//handle cruisecontrol
		CruiseControl();


		//handle undervoltage
		if(fuelcell.undervoltage && fuelcell.fuelcell_startup == 3){

			TurnBuzzerOn(100);			

			//turn on fan if fan is off
			if(signals_lights.green_one == 0){
				panel_buttons_Bluetooth |= 1<<3;
			}

			//turn off load and h2 supply if autooff is selected in menu
			if(fuelcell.load_auto_off){
				send.id = 0x6C4;			
				send.length = 1;
				send.rbr = 0;
				send.rb0 = 0;
				send.data[0] = 0;
				send_can_frame(&send);
			}

		
		}
		fuelcell.undervoltage = 0;
		

		//find fuelcell voltage
		fuelcell.voltage_sum = 0;
		for(i = 0;i<46;i++){
			fuelcell.voltage_sum += ((double) fuelcell.cell_voltage[i])*0.0015;
			
		}

	


		//quick hack uart-debug commands, mostly changing LED-color, useful for testpurposes

		data = USART_Receive_debug(NULL);

		switch (data){
		case'1':
			if(OCR3AL>0){
				OCR3AL--;
			}

		break;
		case'2':
			if(OCR3AL<0xff){
				OCR3AL++;
			}
		break;
		case'3':
			if(OCR3BL>0){
				OCR3BL--;
			}
		break;
		case'4':
			if(OCR3BL<0xff){
				OCR3BL++;
			}
		break;
		case'5':
			if(OCR3CL>0){
				OCR3CL--;
			}
		break;
		case'6':
			if(OCR3CL<0xff){
				OCR3CL++;
			}
		break;
		case'7':
		
			
		break;
		case'r':
			if(!signals_lights.emergency){
				if(signals_lights.left_indicator_light_active == 0){
					signals_lights.left_indicator_light_active = 1;
					signals_lights.right_indicator_light_active = 0;
				}
				else{
					signals_lights.left_indicator_light_active = 0;
				}
				signals_lights.values_changed = 1;
			}

		break;
		case't':
			if(!signals_lights.emergency){
				if(signals_lights.right_indicator_light_active == 0){
					signals_lights.right_indicator_light_active = 1;
					signals_lights.left_indicator_light_active = 0;
				}
				else{
					signals_lights.right_indicator_light_active = 0;
			
				}
				signals_lights.values_changed = 1;
			}
		break;
		case'l':
			if(signals_lights.low_lights_on == 0){
				signals_lights.low_lights_on = 1;

			}
			else{
				signals_lights.low_lights_on = 0;
			}
			signals_lights.values_changed = 1;
		break;
		case'h':
			if(signals_lights.high_lights_on == 0){
				signals_lights.high_lights_on = 1;
			}
			else{
				signals_lights.high_lights_on = 0;
			}
			signals_lights.values_changed = 1;
		break;
		case'e':
			if(signals_lights.emergency == 0){
				signals_lights.emergency = 1;
				signals_lights.right_indicator_light_active = 1;
				signals_lights.left_indicator_light_active = 1;
			}
			else{
				signals_lights.emergency = 0;
				signals_lights.right_indicator_light_active = 0;
				signals_lights.left_indicator_light_active = 0;
			}
			signals_lights.values_changed = 1;
		break;
		case'q':
			input = -1;
		break;
		case'a':
			input = 1;
		break;
		case's':
			enc_push = 1;
		break;
		case'b':
			lcd_blinking_box(1);
		break;
		case'n':
			lcd_blinking_box(0);
		break;
		case'w':
			set_status_led(LED_OFF);
		break;
	
		case'0':
			wdt_enable(WDTO_15MS);
			while(1); //wait for watchdog

		break;
	
		}


		//send and receive CAN-messages

	
		while(new_can_frame()){//receive all new can-frames
			recv = read_can_frame();
			next_can_frame();

			//reset module if requested from CAN-message for bootloaderapplication
			/*
			if(recv->id == 0x000 && recv->data[0] == 0xff){
				wdt_enable(WDTO_15MS);
				while(1); //wait for watchdog
			}
			*/
			switch(recv->id){
				case 0x740:
					switch (recv->data[0]){
						case 1:
							OCR3AL = 0x00;
						break;
						case 2:
							OCR3AL = 0xff;
						break;
						case 3:
							fast_breathing(1);
						break;
						case 4:
							set_status_led(RED);
						break;
						case 5:
							set_status_led(GREEN);
						break;
						case 6:
							set_status_led(BLUE);
						break;
						case 7:
							set_status_led(YELLOW);
						break;
						case 8:
							set_status_led(WHITE);
						break;	
						case 9:
							set_status_led(MAGENTA);
						break;									
						case 10:
							set_status_led(CYAN);
						break;
						case 11:
							set_status_led(ORANGE);
						break;
						case 12:
							set_status_led(LED_OFF);
						break;

					}
				
				break;
			
				case 0x741:
				break;
					
				case 0x742:
					signals_lights.pedal_position = 59-recv->data[0];
			
				break;
			
				case 0x750:
				
					module_status.fuel_cell = recv->data[0];
					ResetModuleStatus(1);
				break;
				case 0x754:
				
					module_status.front_light = recv->data[0];
					ResetModuleStatus(2);
				break;
				case 0x758:
				
					module_status.back_light = recv->data[0];
					ResetModuleStatus(3);
				break;
				case 0x762:
				
					module_status.telemetry = recv->data[0];
					ResetModuleStatus(4);
			
				break;
				case 0x763:
				
					rtc_struct.Hours = recv->data[2];
					rtc_struct.Minutes = recv->data[3];
					rtc_struct.Seconds = recv->data[4];

					rtc_struct.Day = recv->data[5];
					rtc_struct.Month = recv->data[6];
					rtc_struct.Year = recv->data[7];

			
				break;

				case 0x700:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i] |= recv->data[i*2+1];
					}
				break;

				case 0x701:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+4] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+4] |= recv->data[i*2+1];
					}
				break;

				case 0x702:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+8] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+8] |= recv->data[i*2+1];
					}
				break;

				case 0x703:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+12] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+12] |= recv->data[i*2+1];
					}
				break;		

				case 0x704:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+16] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+16] |= recv->data[i*2+1];
					}
				break;

				case 0x705:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+20] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+20] |= recv->data[i*2+1];
					}
				break;

				case 0x706:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+24] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+24] |= recv->data[i*2+1];
					}
				break;

				case 0x707:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+28] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+28] |= recv->data[i*2+1];
					}
				break;

				case 0x708:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+32] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+32] |= recv->data[i*2+1];
					}
				break;

				case 0x709:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+36] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+36] |= recv->data[i*2+1];
					}
				break;

				case 0x70A:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+40] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+40] |= recv->data[i*2+1];
					}
				break;

				case 0x70B:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+44] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+44] |= recv->data[i*2+1];
					}
				break;

				case 0x70C:
					for(i = 0;i<4;i++){
						fuelcell.temperature[i] = recv->data[i*2]<<8;
						fuelcell.temperature[i] |= recv->data[i*2+1];
					}
				break;

				case 0x70D:
					for(i = 0;i<4;i++){
						fuelcell.temperature[i+4] = recv->data[i*2]<<8;
						fuelcell.temperature[i+4] |= recv->data[i*2+1];
					}
				break;

				case 0x70E:
					for(i = 0;i<4;i++){
						fuelcell.temperature[i+8] = recv->data[i*2]<<8;
						fuelcell.temperature[i+8] |= recv->data[i*2+1];
					}
	
				break;

				case 0x70F:

					fuelcell.pressure_sensor[0] = recv->data[0]<<8;
					fuelcell.pressure_sensor[0] |= recv->data[1];
					fuelcell.pressure_sensor[1] = recv->data[2]<<8;
					fuelcell.pressure_sensor[1] |= recv->data[3];
					fuelcell.pressure_sensor[2] = recv->data[4]<<8;
					fuelcell.pressure_sensor[2] |= recv->data[5];


				break;

				case 0x710:
				
					for(i = 0;i<5;i++){
						fuelcell.output[i] = recv->data[i];
					}
					
				break;
				case 0x745:
				
				if(recv->data[0]){//lowbeam
					panel_buttons_Bluetooth |= 1<<0;
				}
				if(recv->data[1]){//highbeam
					panel_buttons_Bluetooth |= 1<<1;
				}
				if(recv->data[2]){//emergency
					panel_buttons_Bluetooth |= 1<<2;
				}
				if(recv->data[3]){
					//showmode
					if(signals_lights.showMode == 3){
						signals_lights.showMode = 0;											
					}else if(signals_lights.showMode == 0){
						signals_lights.showMode = 1;
						slow_breathing(1);		
					}else if(signals_lights.showMode == 1){
						signals_lights.showMode = 2;
						fast_breathing(1);
					}else if(signals_lights.showMode == 2){
						signals_lights.showMode = 3;
						heart_beat(1);
					}
				}
				if(recv->data[4]){
					//horn
					Timer0RegisterCallback(SoundHorn, HORN_PERIOD);
				}					
					
				break;
				case 0x743:
					fuelcell.undervoltage = 1;					
					
				break;
			
			}
			
		}
	



		//send msg over CAN
		SendStatusMsg(0x0c);
		
		
		

		_DELAY_MS(10);
	}
}//end of main




void SendStatusMsg(unsigned char status){
		
	static unsigned int last_system_ticks = 0;
	if(last_system_ticks != get_system_ticks()){//do this every half second
		union can_frame send;
		send.id = 0x7D8;			
		send.length = 1;
		send.rbr = 0;
		send.rb0 = 0;
		send.data[0] = status;
		send_can_frame(&send);

		//battery voltage to motormodule
		send.id = 0x6D0;			
		send.length = 2;
			
		send.data[0] = board_status.input_voltage>>8;
		send.data[1] = board_status.input_voltage & 0x00ff;
		send_can_frame(&send);
			
		last_system_ticks = get_system_ticks();
	}

	//do this every loop of main-loop

	if(signals_lights.purge_fuelcell){
		signals_lights.purge_fuelcell = 0;

		union can_frame send;
		send.id = 0x3C0;			
		send.length = 1;
		send.rbr = 0;
		send.rb0 = 0;
		send.data[0] = 1;
		send_can_frame(&send);
	
	}


	
}

void sendCANmsgLights(void){

	if(signals_lights.values_changed){
		union can_frame send;
		send.id = 0x5C1;			
		send.length = 8;
		send.rbr = 0;
		send.rb0 = 0;

		if(signals_lights.low_lights_on){
			send.data[0] = signals_lights.low_lights_brightness;
		}else{
			send.data[0] = 0;
		}

		if(signals_lights.high_lights_on){

			send.data[1] = signals_lights.high_lights_brightness;
		}else{
		
			send.data[1] = 0;
		}
		if(signals_lights.left_indicator_light_on){
			send.data[2] = signals_lights.left_indicator_light_brightness;
		}else{
			send.data[2] = 0;
		}
		if(signals_lights.right_indicator_light_on){
			send.data[3] = signals_lights.right_indicator_light_brightness;
		}
		else{
			send.data[3] = 0;
		}

		send_can_frame(&send);

		send.id = 0x6C1;			
		send.length = 8;
		send.rbr = 0;
		send.rb0 = 0;


		send.data[0] = signals_lights.backlight_brightness;
		send.data[1] = signals_lights.brakelight_brightness;

		send_can_frame(&send);



		signals_lights.values_changed = 0;

	}
}

void SoundHorn(void){
	static unsigned char horn_on = 0;
	union can_frame send;
	send.id = 0x5c0;			
	send.length = 1;
	send.rbr = 0;
	send.rb0 = 0;

	if(!horn_on){
		send.data[0] = 1;
		horn_on = 1;
	}
	else{
		send.data[0] = 0;
		horn_on = 0;
		Timer0RemoveCallback(SoundHorn);
	}
	send_can_frame(&send);	
	
}


void IndicatorLights(void){

	

	if(signals_lights.left_indicator_light_active || signals_lights.right_indicator_light_active){
		
		if(signals_lights.IndicatorTickerOn){
			PlayClick();
		}
		if(signals_lights.left_indicator_light_active){
			if(signals_lights.left_indicator_light_on == 0){
				signals_lights.left_indicator_light_on = 1;
			}else{
				signals_lights.left_indicator_light_on = 0;
			}
	
		}
		else{
			signals_lights.left_indicator_light_on = 0;
		
		}


		if(signals_lights.right_indicator_light_active){
			if(signals_lights.right_indicator_light_on == 0){
				signals_lights.right_indicator_light_on = 1;					
			}
			else{
				signals_lights.right_indicator_light_on = 0;
			}
		}
		else{
			signals_lights.right_indicator_light_on = 0;
		
		}

	
		
		
	}else{

		signals_lights.right_indicator_light_on = 0;
		signals_lights.left_indicator_light_on = 0;

		Timer0RemoveCallback(IndicatorLights);
	}
	signals_lights.values_changed = 1;
}	

void SetTickerPeriod(int input){
	signals_lights.ticker_period = (unsigned char)input;
	Timer0RemoveCallback(IndicatorLights);
	
	Timer0RegisterCallback(IndicatorLights, signals_lights.ticker_period);
}


void ResetModuleStatus(int reset_node_number){
	
	static unsigned int last_system_ticks = 0;
	static unsigned char counter[5] = {0};	

	
	counter[reset_node_number] = 0;
	
	
	if(get_system_ticks() != last_system_ticks){
		
		int i;
		for(i = 0;i<5;i++){
			counter[i]++;
		
			if(counter[1]>2){
				module_status.fuel_cell = 0;
				counter[1] = 0;
			}
			if(counter[2]>2){
				module_status.front_light = 0;
				counter[2] = 0;
			}
			if(counter[3]>2){
				module_status.back_light = 0;
				counter[3] = 0;
			}
			if(counter[4]>2){
				module_status.telemetry = 0;
				counter[4] = 0;
			}

			
		}

		last_system_ticks = get_system_ticks();
	}

}

void HandleButtonPushes(unsigned char panel_buttons, unsigned char wheel_buttons){
	/* panel_buttons_pushed
	bit0 LowLights
	bit1 HighLights
	bit2 emBlink

	bit3 green1
	bit4 green2
	bit5 black1
 
 	wheel_buttons_pushed
	bit0 horn
	bit1 left blink
	bit2 right blink

	bit3 decrease cruise
	bit4 increase cruise
	*/

	//LowLights
	if(TEST_BIT(panel_buttons, 0)){
		if(signals_lights.low_lights_on == 0){
			signals_lights.low_lights_on = 1;

		}
		else{
			signals_lights.low_lights_on = 0;
		}
		signals_lights.values_changed = 1;	
	}
	//HighLights
	if(TEST_BIT(panel_buttons, 1)){
		if(signals_lights.high_lights_on == 0){
			signals_lights.high_lights_on = 1;
		}
		else{
			signals_lights.high_lights_on = 0;
		}
		signals_lights.values_changed = 1;	
	
	}
	//emBlink
	if(TEST_BIT(panel_buttons, 2)){
		if(signals_lights.emergency == 0){
			Timer0RegisterCallback(IndicatorLights, signals_lights.ticker_period);
			signals_lights.emergency = 1;
			signals_lights.right_indicator_light_active = 1;
			signals_lights.left_indicator_light_active = 1;
		}
		else{
			signals_lights.emergency = 0;
			signals_lights.right_indicator_light_active = 0;
			signals_lights.left_indicator_light_active = 0;
		}
		signals_lights.values_changed = 1;
	}
	//green1 coast 
	if(TEST_BIT(panel_buttons, 3)){
	
		union can_frame send;
		send.id = 0x6C0;			
		send.length = 1;
		send.rbr = 0;
		send.rb0 = 0;
		send.data[0] = 1;
		send_can_frame(&send);
		_DELAY_MS(10);

		send.data[0] = 0;
		send_can_frame(&send);
	}
	//green2 turn fuelcell on/off
	if(TEST_BIT(panel_buttons, 4)){
		if(signals_lights.green_two == 0){
			TurnBuzzerOn(60);
			signals_lights.green_two = 1;
			fuelcell.fuelcell_startup = 1;
			//to motormodule, load relay
			union can_frame send;
			send.id = 0x6C4;			
			send.length = 1;
			send.rbr = 0;
			send.rb0 = 0;
			send.data[0] = 1;
			send_can_frame(&send);
			//to fuelcell
			send.id = 0x3C1; 
			send_can_frame(&send);
			Timer0RegisterCallback(FuelCellStartedCallback, 230);
		}
		else{
			signals_lights.green_two = 0;
			//to motormodule, load relay
			union can_frame send;
			send.id = 0x6C4;			
			send.length = 1;
			send.rbr = 0;
			send.rb0 = 0;
			send.data[0] = 0;
			send_can_frame(&send);
			//to fuelcell
			send.id = 0x3C1; 
			send_can_frame(&send);

		}
		signals_lights.values_changed = 1;	
	
	}
	//black1
	if(TEST_BIT(panel_buttons, 5)){
		
		
		CruiseOnPressed();
	
	}

	
	
	//Left indicator
	if(TEST_BIT(wheel_buttons, 0)){
		if(signals_lights.emergency == 0){
			if(signals_lights.left_indicator_light_active == 0){
				signals_lights.left_indicator_light_active = 1;
				signals_lights.right_indicator_light_active = 0;
			}
			else{
				signals_lights.left_indicator_light_active = 0;
			}

			signals_lights.values_changed = 1;
			Timer0RegisterCallback(IndicatorLights, signals_lights.ticker_period);
		}
	}

	//cruise--
	if(TEST_BIT(wheel_buttons, 1)){
		DecreaseCruiseSpeed();
	}
	//horn
	if(TEST_BIT(wheel_buttons, 2)){
	
		
	}
	//right indicator
	if(TEST_BIT(wheel_buttons, 3)){
		if(signals_lights.emergency == 0){
			if(signals_lights.right_indicator_light_active == 0){
				signals_lights.right_indicator_light_active = 1;
				signals_lights.left_indicator_light_active = 0;
			}
			else{
				signals_lights.right_indicator_light_active = 0;
			}

			signals_lights.values_changed = 1;
			Timer0RegisterCallback(IndicatorLights, signals_lights.ticker_period);
		}
		
	}	
	//cruise++
	if(TEST_BIT(wheel_buttons, 4)){
		IncreaseCruiseSpeed();
	}
}


void HandleButtonLED(void){

	if(signals_lights.low_lights_on){
		set_io_bit('B', 7);
	}else{
		clear_io_bit('B', 7);
	}

	if(signals_lights.high_lights_on){
		set_io_bit('B', 6);
	}else{
		clear_io_bit('B', 6);
	}
	if(signals_lights.left_indicator_light_on && signals_lights.right_indicator_light_on && signals_lights.emergency){
		set_io_bit('B', 5);
	}else{
		clear_io_bit('B', 5);
	}
	if(signals_lights.green_one){
		set_io_bit('B', 4);
	}else{
		clear_io_bit('B', 4);
	}
	if(signals_lights.green_two){
		set_io_bit('B', 3);
	}else{
		clear_io_bit('B', 3);
	}
	
	if(signals_lights.black_one){
		set_io_bit('A', 5);
	}else{
		clear_io_bit('A', 5);
	}
	
}


void FanCallback(void){
	static unsigned char seconds = 0;

	seconds++;
	if(seconds>fuelcell.fan_fast_time){
		union can_frame send;
		send.id = 0x3C2;			
		send.length = 1;
		send.rbr = 0;
		send.rb0 = 0;
		send.data[0] = fuelcell.fan_dutycycle_slow;
		send_can_frame(&send);
		seconds = 0;
		Timer0RemoveCallback(FanCallback);
	}


}



void setFanSlowSpeed(int input){

	fuelcell.fan_dutycycle_slow = (unsigned char) input;
}

void setFanFastSpeed(int input){
	fuelcell.fan_dutycycle_fast = (unsigned char) input;

}

void setLoadAutoOff(int input){

	fuelcell.load_auto_off = input;
}
void setFanFastTime(int input){

	fuelcell.fan_fast_time = input;
}

void FuelCellStartedCallback(void){
	fuelcell.fuelcell_startup = 2;


	Timer0RemoveCallback(FuelCellStartedCallback);
}
