
/* rev 1.0 02/2010 				written by Anders Guldahl			     */

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "macro.h"
#include "SPI_routines.h"
#include "mcp23S17.h"
#include "can.h"

const prog_uint8_t ace_tab[] = {0,56,40,55,
24,0,39,52,8,57,0,0,23,0,36,13,120,0,41,54,
0,0,0,53,7,0,0,0,20,19,125,18,104,105,0,0,
25,106,38,0,0,58,0,0,0,0,37,14,119,118,0,0,
0,107,0,0,4,0,3,0,109,108,2,1,88,0,89,0,
0,0,0,51,9,10,90,0,22,11,0,12,0,0,42,43,
0,0,0,0,0,0,0,0,21,0,126,127,103,0,102,0,
0,0,0,0,0,0,91,0,0,0,0,0,116,117,0,0,115,
0,0,0,93,94,92,0,114,95,113,0,72,71,0,68,73,
0,0,29,0,70,0,69,0,0,35,34,121,0,122,0,74,0,
0,30,6,0,123,0,0,0,124,17,0,0,0,67,26,0,
27,28,0,59,0,0,0,0,0,15,0,0,0,0,0,0,0,
0,5,0,0,0,110,0,111,16,87,84,0,45,86,85,
0,50,0,0,0,46,0,0,0,33,0,83,0,44,75,0,
0,31,0,0,0,0,0,0,0,32,100,61,101,
66,0,62,0,49,99,60,0,47,0,0,0,48,77,82,78,
65,76,63,0,64,98,81,79,80,97,96,112
};

volatile int rotations = 0;
volatile unsigned char enc_push = 0;

/* panel_buttons_pushed
bit0 LowLights
bit1 high lights
bit2 em blink

bit3 green1
bit4 green2
bit5 black1
*/
volatile unsigned char panel_buttons_pushed = 0; 

/* wheel_buttons_pushed
bit0 horn
bit1 left blink
bit2 right blink

bit3 decrease cruise
bit4 increase cruise
*/
volatile unsigned char wheel_buttons_pushed = 0;

unsigned char init_io_expander(void){

	
	SW_IO_CS_ASSERT;

	SPI_transmit(0x40);	//write mode
	SPI_transmit(0x00); //start at register 0x00

	SPI_transmit(0xff); //reg:0x00 Port A input
	SPI_transmit(0xff); //reg:0x01 Port B input
	SPI_transmit(0x00); //reg:0x02 polarityA
	SPI_transmit(0x00); //reg:0x03 polarityB
	SPI_transmit(0xC7); //reg:0x04 INT A enabled on encoder-pins 0,1,2, and switchpins 6, 7
	SPI_transmit(0xff); //reg:0x05 INT B enabled on all switchpins 0 - 7
	SPI_transmit(0x00); //reg:0x06 DEFVALA
	SPI_transmit(0x00); //reg:0x07 DEFVALB
	SPI_transmit(0x00); //reg:0x08 INTCONA //interrupt on pin-change is active 0
	SPI_transmit(0x00); //reg:0x09 INTCONB	//interrupt on pin-change is active 0
	SPI_transmit(0x04); //reg:0x0A IOCON	
	SPI_transmit(0x04); //reg:0x0B IOCON	//same as above?
	SPI_transmit(0x00); //reg:0x0C PULLUPA
	SPI_transmit(0x00); //reg:0x0D PULLUPB
	SPI_transmit(0x00); //reg:0x0E INTFLAG A
	SPI_transmit(0x00); //reg:0x0F INTFLAG B
	SPI_transmit(0x00); //reg:0x10 INT CAPTURE A
	SPI_transmit(0x00); //reg:0x11 INT CAPTURE B
	SPI_transmit(0x00); //reg:0x12 GPIOA
	SPI_transmit(0x00); //reg:0x13 GPIOB
	SPI_transmit(0x00); //reg:0x14 OUTPUT LATCH A
	SPI_transmit(0x00); //reg:0x15 OUTPUT LATCH B
	
	
	SW_IO_CS_DEASSERT;


	//io expander for sw10 and sw11 and leds
	LED_IO_CS_ASSERT;

	SPI_transmit(0x40);	//write mode
	SPI_transmit(0x00); //start at register 0x00

	SPI_transmit(0x03); //reg:0x00 Port A 0 and 1 input, rest output
	SPI_transmit(0x00); //reg:0x01 Port B output
	SPI_transmit(0x00); //reg:0x02 polarityA
	SPI_transmit(0x00); //reg:0x03 polarityB
	SPI_transmit(0x03); //reg:0x04 INT A enabled on switch-pins 0,1
	SPI_transmit(0x00); //reg:0x05 
	SPI_transmit(0x00); //reg:0x06 DEFVALA
	SPI_transmit(0x00); //reg:0x07 DEFVALB
	SPI_transmit(0x00); //reg:0x08 INTCONA //interrupt on pin-change is active 0
	SPI_transmit(0x00); //reg:0x09 INTCONB	//interrupt on pin-change is active 0
	SPI_transmit(0x04); //reg:0x0A IOCON	
	SPI_transmit(0x04); //reg:0x0B IOCON	//same as above?
	SPI_transmit(0x00); //reg:0x0C PULLUPA
	SPI_transmit(0x00); //reg:0x0D PULLUPB
	SPI_transmit(0x00); //reg:0x0E INTFLAG A
	SPI_transmit(0x00); //reg:0x0F INTFLAG B
	SPI_transmit(0x00); //reg:0x10 INT CAPTURE A
	SPI_transmit(0x00); //reg:0x11 INT CAPTURE B
	SPI_transmit(0x00); //reg:0x12 GPIOA
	SPI_transmit(0x00); //reg:0x13 GPIOB
	SPI_transmit(0x00); //reg:0x14 OUTPUT LATCH A
	SPI_transmit(0x00); //reg:0x15 OUTPUT LATCH B
	
	
	LED_IO_CS_DEASSERT;


	//init io_B interruptpin on avr pin int0, enable pull-up, trigger on falling edge
	DDRD &= ~(1<<PD0); //set as input
	PORTD |= (1<<PD0); //enable pullup

	EICRA &= ~((1<<ISC01)|(1<<ISC00)); //low level interrupt
	
	//init io_A interruptpin on avr pin int1, enable pull-up, trigger on falling edge
	DDRD &= ~(1<<PD1); //set as input
	PORTD |= (1<<PD1); //enable pullup

	EICRA &= ~((1<<ISC11)|(1<<ISC10)); //low level interrupt

	//init io_A expander 2 interruptpin on avr pin int0, enable pull-up, trigger on falling edge
	DDRD &= ~(1<<PD2); //set as input
	PORTD |= (1<<PD2); //enable pullup

	EICRA &= ~((1<<ISC21)|(1<<ISC20)); //low level interrupt

	ENABLE_EXT_INTERRUPT; //enable interrupts

	return 0;
}


unsigned char set_io_bit(char port, unsigned char pin){
	
	DISABLE_EXT_INTERRUPT;
	if(port == 'A'){
		LED_IO_CS_ASSERT;
		SPI_transmit(0x41);	//read mode
		SPI_transmit(0x14); //register 0x14 OLATA
		unsigned char port = SPI_receive();
		LED_IO_CS_DEASSERT;

		port |= (1<<pin); // set bit
		
		//write back
		LED_IO_CS_ASSERT;
		SPI_transmit(0x40);	//write mode
		SPI_transmit(0x14); //register 0x14 OLATA
		SPI_transmit(port);
		LED_IO_CS_DEASSERT;

	
	}
	else if(port == 'B'){
		LED_IO_CS_ASSERT;
		SPI_transmit(0x41);	//read mode
		SPI_transmit(0x15); //register 0x15 OLATB
		unsigned char port = SPI_receive();
		LED_IO_CS_DEASSERT;

		port |= (1<<pin); // set bit
		
		//write back
		LED_IO_CS_ASSERT;
		SPI_transmit(0x40);	//write mode
		SPI_transmit(0x15); //register 0x15 OLATB
		SPI_transmit(port);
		LED_IO_CS_DEASSERT;


	}
	ENABLE_EXT_INTERRUPT;
	return 0;
}

unsigned char clear_io_bit(char port, unsigned char pin){
	DISABLE_EXT_INTERRUPT;

	if(port == 'A'){
		LED_IO_CS_ASSERT;
		SPI_transmit(0x41);	//read mode
		SPI_transmit(0x14); //register 0x14 OLATA
		unsigned char port = SPI_receive();
		LED_IO_CS_DEASSERT;

		port &= ~(1<<pin); // clear bit
		
		//write back
		LED_IO_CS_ASSERT;
		SPI_transmit(0x40);	//write mode
		SPI_transmit(0x14); //register 0x14 OLATA
		SPI_transmit(port);
		LED_IO_CS_DEASSERT;

	
	}
	else if(port == 'B'){
		LED_IO_CS_ASSERT;
		SPI_transmit(0x41);	//read mode
		SPI_transmit(0x15); //register 0x15 OLATB
		unsigned char port = SPI_receive();
		LED_IO_CS_DEASSERT;

		port &= ~(1<<pin); // clear bit
		
		//write back
		LED_IO_CS_ASSERT;
		SPI_transmit(0x40);	//write mode
		SPI_transmit(0x15); //register 0x15 OLATB
		SPI_transmit(port);
		LED_IO_CS_DEASSERT;


	}
	ENABLE_EXT_INTERRUPT;
	return 0;
}




int get_encoder_rotations(void){
	return rotations;
}

void reset_encoder_rotations(void){
	rotations = 0;

}

unsigned char get_enc_push(void){
	return enc_push;
}

void reset_enc_push(void){
	enc_push = 0;
}

unsigned char getPanelButtons(void){
	return panel_buttons_pushed;
}

void resetPanelButtons(void){
	panel_buttons_pushed = 0;
}

unsigned char getWheelButtons(void){
	return wheel_buttons_pushed;
}

void resetWheelButtons(void){
	wheel_buttons_pushed = 0;
}

ISR(INT0_vect){
	
	SW_IO_CS_ASSERT;
	//read INTCAPB
	SPI_transmit(0x41);	//read mode
	SPI_transmit(0x0F); //register 0x0F INTFLAGB
	unsigned char int_flag = SPI_receive();
	SPI_receive();	//dummy intcapA
	unsigned char io_reg = SPI_receive(); //intcapB read
	SW_IO_CS_DEASSERT;


	/* panel_buttons_pushed
	bit0 LowLights
	bit1 high lights
	bit2 em blink

	bit3 green1
	bit4 green2

	*/

	if(TEST_BIT(int_flag,0)){//switch 2 caused interrupt em_blink
		if(TEST_BIT(io_reg, 0)){ //released
		

		}
		else{ //pushed
			panel_buttons_pushed |= (1<<2);
		
		}

	}
	if(TEST_BIT(int_flag,1)){//switch 3 caused interrupt green1
		if(TEST_BIT(io_reg, 1)){ //released
		

		}
		else{ //pushed
			
			panel_buttons_pushed |= (1<<3);
		}

	}
	if(TEST_BIT(int_flag,2)){//switch 4 caused interrupt green2
		if(TEST_BIT(io_reg, 2)){ //released
		

		}
		else{ //pushed
			
			panel_buttons_pushed |= (1<<4);
		}

	}


	if(TEST_BIT(int_flag,3)){//switch 5 caused interrupt Left indikator
		if(TEST_BIT(io_reg, 3)){ //released
		

		}
		else{ //pushed
			
			wheel_buttons_pushed |= (1<<0);
		}

	}
	if(TEST_BIT(int_flag,4)){//switch 6 caused interrupt cruise --
		if(TEST_BIT(io_reg, 4)){ //released
		

		}
		else{ //pushed
			
			wheel_buttons_pushed |= (1<<1);
		}

	}
	if(TEST_BIT(int_flag,5)){//switch 7 caused interrupt horn
		if(TEST_BIT(io_reg, 5)){ //released
			//horn off
			union can_frame send;
			send.id = 0x5c0;			
			send.length = 1;
			send.rbr = 0;
			send.rb0 = 0;
			send.data[0] = 0;
			send_can_frame(&send);

		}
		else{ //pushed
			//horn on
			union can_frame send;
			send.id = 0x5c0;			
			send.length = 1;
			send.rbr = 0;
			send.rb0 = 0;
			send.data[0] = 1;
			send_can_frame(&send);
		}

	}

	if(TEST_BIT(int_flag,6)){//switch 8 caused interrupt right indikator
		if(TEST_BIT(io_reg, 6)){ //released


		}
		else{ //pushed


			wheel_buttons_pushed |= (1<<3);
		}

	}
	if(TEST_BIT(int_flag,7)){//switch 9 caused interrupt cruise ++
		if(TEST_BIT(io_reg, 7)){ //released

		}
		else{ //pushed
	
			wheel_buttons_pushed |= (1<<4);
		}

	}
}

volatile unsigned char state = 0;
volatile signed char state_counter = 0;


ISR(INT1_vect){
	
	

	SW_IO_CS_ASSERT;
	
	//read INTCAPA
	SPI_transmit(0x41);	//read mode
	SPI_transmit(0x0E); //register 0x0E INTFLAGA
	unsigned char int_flag = SPI_receive();
	SPI_receive();	//dummy intflagB
	unsigned char io_reg = SPI_receive(); //intcapA read
	SW_IO_CS_DEASSERT;
	
	if(TEST_BIT(int_flag,0) || TEST_BIT(int_flag,1)){ //encoder caused interrupt
	

		//update statemachine
		switch(state){
			case 0:
							
				if(TEST_BIT(io_reg, 0) == 0){
					state_counter++;
					state = 1;
				}
				else if(TEST_BIT(io_reg, 1) == 0){
					state_counter--;
					state = 3;
				}

			break;
			case 1:
				if(TEST_BIT(io_reg, 1) == 0){
					state = 2;
					state_counter++;							
				}
				else if(TEST_BIT(io_reg, 0) == 1){
					state = 0;
					state_counter--;
					if(state_counter <= -4){
						rotations++;
						state_counter = 0;
					}
				}

			break;
			case 2:
				if(TEST_BIT(io_reg, 0) == 1){
					state = 3;
					state_counter++;
				}
				else if(TEST_BIT(io_reg, 1) == 1){
					state = 1;
					state_counter--;
				}

			break;
			case 3:
				if(TEST_BIT(io_reg, 1) == 1){
					state = 0;
					state_counter++;
			    	if(state_counter >= 4){
						rotations--;
						state_counter = 0;
					}			
				}
				else if(TEST_BIT(io_reg, 0) == 0){
					state = 2;
					state_counter--;		
				}

			break;	
	
		}
	}

	if(TEST_BIT(int_flag,2)){//encoder switch 1 caused interrupt
		if(TEST_BIT(io_reg, 2)){ //released
		

		}
		else{ //pushed
			enc_push = 1;
		
		}

	}
	/* panel_buttons_pushed
	bit0 LowLights
	bit1 high lights

	*/
	if(TEST_BIT(int_flag,6)){//switch 0 caused interrupt low lights
		if(TEST_BIT(io_reg, 6)){ //released
		

		}
		else{ //pushed
			panel_buttons_pushed |= (1<<0);
		
		}

	}
	if(TEST_BIT(int_flag,7)){//switch 1 caused interrupt high lights
		if(TEST_BIT(io_reg, 7)){ //released
		

		}
		else{ //pushed
			
			panel_buttons_pushed |= (1<<1);
		}

	}
}

ISR(INT2_vect){
	
	LED_IO_CS_ASSERT;
	//read INTCAPA
	SPI_transmit(0x41);	//read mode
	SPI_transmit(0x0E); //register 0x0E INTFLAGA
	unsigned char int_flag = SPI_receive();
	SPI_receive();	//dummy intflagB
	unsigned char io_reg = SPI_receive(); //intcapA read
	LED_IO_CS_DEASSERT;


	/* panel_buttons_pushed

	bit5 black1
	*/
	if(TEST_BIT(int_flag,0)){//switch 10 caused interrupt, black panel button cruisecontrol on/off
		if(TEST_BIT(io_reg, 0)){ //released
			
		}
		else{ //pushed
			panel_buttons_pushed |= (1<<5);			
		}

	}
	if(TEST_BIT(int_flag,1)){//switch 11 caused interrupt
		if(TEST_BIT(io_reg, 1)){ //released
		

		}
		else{ //pushed
			
			
		}

	}

}
