

#include <stdio.h>
#include <string.h>
#include <inttypes.h>
#include <stdlib.h>
#include "uart_lcd.h"
#include "menu.h"
#include "main.h"
#include "uart_debug.h"
#include "../common_files/status_structs.h"
#include "lights.h"
#include "status_led.h"
#include "cruise.h"




const char MT_MAIN_STATUS_ONE[] PROGMEM                 = "1  Drive";
const char MT_MAIN_STATUS_TWO[] PROGMEM                 = "2  Status";
const char MT_MAIN_FUELCELL[] PROGMEM 	                = "3  FuelCell";
const char MT_MAIN_MOTOR[] PROGMEM	                    = "4  Motor status";
const char MT_MAIN_GPS[] PROGMEM	                    = "5  GPS";
const char MT_MAIN_OPTIONS[] PROGMEM                	= "6  Options";
const char MT_MAIN_LIGHTS[] PROGMEM                		= "7  Lights";
const char MT_MAIN_LOG[] PROGMEM                		= "8  Log";
const char MT_MAIN_DEBUG[] PROGMEM                		= "9  Debug mode";
const char MT_MAIN_POWERSAVE[] PROGMEM                  = "10 Powersave";
const char MT_MAIN_MATRIX[] PROGMEM                 	= "11 Matrix";
const char MT_MAIN_PURGE[] PROGMEM                 		= "12 Purge";


const char MT_OPTIONS_LCD_BRIGHTNESS[] PROGMEM          = "1 LCD Backlight";
const char MT_OPTIONS_TICKER[] PROGMEM           		= "2 Tickersound";
const char MT_OPTIONS_TICKER_PERIOD[] PROGMEM          	= "3 Tickerperiod";
const char MT_OPTIONS_CRUISE_SAFETY[] PROGMEM          	= "4 Cruise Safety";
const char MT_OPTIONS_CRUISE_MAX[] PROGMEM          	= "5 Cruise Max";
const char MT_OPTIONS_FAN_SLOW[] PROGMEM          		= "6 Fan slow";
const char MT_OPTIONS_FAN_FAST[] PROGMEM          		= "7 Fan fast";
const char MT_OPTIONS_FAN_FAST_TIME[] PROGMEM          	= "8 Fan timer s";
const char MT_OPTIONS_LOAD_OFF[] PROGMEM          		= "9 u.v Loadoff";



const char MT_LIGHTS_LOWBEAM[] PROGMEM           		= "1 LowBeam";
const char MT_LIGHTS_HIGHBEAM[] PROGMEM           		= "2 HighBeam";
const char MT_LIGHTS_TURNSIGNALS[] PROGMEM           	= "3 Turnsignals";
const char MT_LIGHTS_BACKLIGHT[] PROGMEM           		= "4 Backlight";
const char MT_LIGHTS_BRAKELIGHT[] PROGMEM           	= "5 Brakelight";

const char MT_BACK[] PROGMEM           					= "  Back";





unsigned char new_menu_refresh = 1;
unsigned char select_refresh = 1;



MAIN_MENU_ELEMENTS main_menu_elements[]= {
// 	show cursor    Text						     	
    {1,           MT_MAIN_STATUS_ONE},
    {0,           MT_MAIN_STATUS_TWO},
    {0,           MT_MAIN_FUELCELL},
	{0,           MT_MAIN_MOTOR},
    {0,           MT_MAIN_GPS},
    {0,           MT_MAIN_OPTIONS},
	{0,           MT_MAIN_LIGHTS},
	{0,           MT_MAIN_LOG},
	{0,           MT_MAIN_DEBUG},
    {0,           MT_MAIN_POWERSAVE},
	{0,			  MT_MAIN_MATRIX},
	{0,			  MT_MAIN_PURGE}
};

#define NUM_MAIN_STATUS_DRIVE 0
#define NUM_MAIN_STATUS_TWO 1
#define NUM_MAIN_FUELCELL 2
#define NUM_MAIN_MOTOR 3
#define NUM_MAIN_GPS 4
#define NUM_MAIN_OPTIONS 5
#define NUM_MAIN_LIGHTS 6
#define NUM_MAIN_LOG 7
#define NUM_MATRIX 10
#define NUM_PURGE 11


SETUP_MENU_ELEMENTS options_menu_elements[]= {
// 	show cursor    text				       	value			value_min	value_max	value_changed   pFunc	
    {1,           MT_OPTIONS_LCD_BRIGHTNESS,    29,			0,			29,				1,			set_lcd_brightness},
    {0,           MT_OPTIONS_TICKER,  		 	1,			0,			1,				1,			SetTickerSound},
	{0,           MT_OPTIONS_TICKER_PERIOD,   	61,			20,			120,			1,			SetTickerPeriod},
	{0,           MT_OPTIONS_CRUISE_SAFETY,   	0,			0,			1,				1,			setCruiseSafety},
	{0,           MT_OPTIONS_CRUISE_MAX,   		90,			0,			100,			1,			setCruiseMaxSpeed},
	{0,           MT_OPTIONS_FAN_SLOW,   		130,		0,			130,			1,			setFanSlowSpeed},
	{0,           MT_OPTIONS_FAN_FAST,   		130,		0,			130,			1,			setFanFastSpeed},	
	{0,           MT_OPTIONS_FAN_FAST_TIME,		5,			1,			100,			1,			setFanFastTime},	
	{0,           MT_OPTIONS_LOAD_OFF,   		0,			0,			1,				1,			setLoadAutoOff},	
		
	{0,           MT_BACK,  	 				-1,			0,			100,			0,			NULL}
};



DRAW_MENU_STATUS options_draw_menu_status = {
//cursor_position		top_position	redraw_needed	redraw_cursor_needed	redraw_value_needed		last_cursor_position
0,0,1,1,1,0
};


SETUP_MENU_ELEMENTS lights_menu_elements[]= {
// 	show cursor    text				       	value			value_min	value_max	value_changed   pFunc
    {1,           MT_LIGHTS_LOWBEAM,   			50,			0,			255,			1,			SetLowBeamBrightness},
    {0,           MT_LIGHTS_HIGHBEAM,   		50,			0,			255,			1,			SetHighBeamBrightness},
	{0,           MT_LIGHTS_TURNSIGNALS,     	50,			0,			60,				1,			SetIndicatorLightBrightness},
    {0,           MT_LIGHTS_BACKLIGHT,   		20,			0,			60,				1,			SetBackLightBrightness},
    {0,           MT_LIGHTS_BRAKELIGHT,  	 	50,			0,			60,				1,			SetBrakeLightBrightness},
	{0,           MT_BACK,  	 				-1,			0,			100,			0,			NULL}
};

DRAW_MENU_STATUS lights_draw_menu_status = {
//cursor_position		top_position	redraw_needed	redraw_cursor_needed	redraw_value_needed		last_cursor_position
0,0,1,1,1,0
};


void HandleOptionMenuChanges(void){
	int i;
	for(i = 0;i<OPTIONS_MENU_LENGTH;i++){
		if(options_menu_elements[i].value_changed && options_menu_elements[i].pFunc != NULL){
			options_menu_elements[i].pFunc(options_menu_elements[i].value);
			options_menu_elements[i].value_changed = 0;
		
		}	
	}
}

void HandleLightsMenuChanges(void){
	int i;
	for(i = 0;i<LIGHTS_MENU_LENGTH;i++){
		if(lights_menu_elements[i].value_changed && lights_menu_elements[i].pFunc != NULL){
			lights_menu_elements[i].pFunc(lights_menu_elements[i].value);
			lights_menu_elements[i].value_changed = 0;
		
		}	
	}
}


int LcdScreenStateMachine(int cursor_movement, unsigned char enc_push){
	
	static int state = ST_MAIN_MENU;
	static int element_selected = 0;
	static int purge_counter = 0;
	int menu_position;


	switch(state){
		
		case ST_MAIN_MENU:
			menu_position = DrawMainMenu(cursor_movement);
			
			if(enc_push){
				switch(menu_position){
					case NUM_MAIN_STATUS_DRIVE:
						state = ST_DRIVE;
						new_menu_refresh = 1;						
					break;
					case NUM_MAIN_STATUS_TWO:
						state = ST_STATUS_TWO;
						new_menu_refresh = 1;						
					break;
					case NUM_MAIN_FUELCELL:
						state = ST_STATUS_FUELCELL;
						new_menu_refresh = 1;						
					break;
					case NUM_MAIN_MOTOR:
						state = ST_STATUS_MOTOR;
						new_menu_refresh = 1;						
					break;
					case NUM_MAIN_GPS:
						state = ST_STATUS_GPS;
						new_menu_refresh = 1;						
					break;										
					case NUM_MAIN_OPTIONS:
						state = ST_OPTIONS_MENU;
						new_menu_refresh = 1;
					break;
					case NUM_MAIN_LIGHTS:
						state = ST_LIGHTS_MENU;
						new_menu_refresh = 1;
					break;
					case NUM_MAIN_LOG:
						state = ST_STATUS_LOG;
						new_menu_refresh = 1;
					break;	
					
					case NUM_MATRIX:
						state = ST_MATRIX;
						new_menu_refresh = 1;
					break;	
					case NUM_PURGE:
						state = ST_PURGE;
					break;	
							
				}
			
			
			}
			

		break;

		case ST_OPTIONS_MENU:
			//draw menu
			menu_position = DrawSetupMenu(cursor_movement, element_selected, options_menu_elements, OPTIONS_MENU_LENGTH, &options_draw_menu_status);
		
			//handle optionsmenu-item changes, for example brightnesschanges
			HandleOptionMenuChanges();

			if(enc_push){
				switch(menu_position){
					case (OPTIONS_MENU_LENGTH - 1):	//go back					
						state = ST_MAIN_MENU;
						new_menu_refresh = 1;
					break;
			
					default:
						if(element_selected){
							element_selected = 0;
						
						}else{
							element_selected = 1;
						}						
						select_refresh = 1;
					break;
				}
			
			
			}
		break;
		case ST_DRIVE:
			DrawDrive(cursor_movement);				

			if(enc_push){
				state = ST_MAIN_MENU;
				new_menu_refresh = 1;			
			}

		break;
		case ST_STATUS_TWO:
			DrawStatus(0);				

			if(enc_push){
				state = ST_MAIN_MENU;
				new_menu_refresh = 1;			
			}

		break;
		case ST_STATUS_FUELCELL:
			DrawFuelCellStatus(0);				

			if(enc_push){
				state = ST_MAIN_MENU;
				new_menu_refresh = 1;			
			}

		break;
		case ST_STATUS_MOTOR:
			DrawMotorStatus(0);				

			if(enc_push){
				state = ST_MAIN_MENU;
				new_menu_refresh = 1;			
			}

		break;
		case ST_STATUS_GPS:
			DrawGpsStatus(0);				

			if(enc_push){
				state = ST_MAIN_MENU;
				new_menu_refresh = 1;			
			}

		break;
		case ST_LIGHTS_MENU:
			//draw menu
			menu_position = DrawSetupMenu(cursor_movement, element_selected, lights_menu_elements, LIGHTS_MENU_LENGTH, &lights_draw_menu_status);

			//handle optionsmenu-item changes, for example brightnesschanges
			HandleLightsMenuChanges();

			if(enc_push){
				switch(menu_position){
					case (LIGHTS_MENU_LENGTH - 1):	//go back					
						state = ST_MAIN_MENU;
						new_menu_refresh = 1;
					break;
			
					default:
						if(element_selected){
							element_selected = 0;
						
						}else{
							element_selected = 1;
						}						
						select_refresh = 1;
					break;
				}
			
			
			}

		break;
		case ST_STATUS_LOG:
			DrawLog(0);				

			if(enc_push){
				state = ST_MAIN_MENU;
				new_menu_refresh = 1;			
			}

		break;
		case ST_MATRIX:
			DrawMatrix(0);				

			if(enc_push){
				state = ST_MAIN_MENU;
				new_menu_refresh = 1;			
			}

		break;
		case ST_PURGE:
			
			if(purge_counter == 0){			
				signals_lights.purge_fuelcell = 1;
			
				LcdSetCursorXY(12,3);
				USART_Transmit_lcd_string("Purging",7);			
			}

			purge_counter++;
			if(purge_counter>20){
				state = ST_MAIN_MENU;
				new_menu_refresh = 1;
				purge_counter = 0;			
			}

		break;	
	}

	

	return 0;
}


int DrawDrive(int input){
	if(new_menu_refresh){
		clear_display();
		new_menu_refresh = 0;
	}	

	//draw turn indicators
	if(signals_lights.left_indicator_light_on){
		DrawSymbolXY(1,0,0x7F);
	}else{
		DrawSymbolXY(1,0,0x20);
	}
	if(signals_lights.right_indicator_light_on){
		DrawSymbolXY(18,0,0x7E);
	}else{
		DrawSymbolXY(18,0,0x20);
	}
	
	if(signals_lights.high_lights_on){
		LcdSetCursorXY(6,1);
		USART_Transmit_lcd_string("(*)",3);
		LcdSetCursorXY(11,1);
		USART_Transmit_lcd_string("(*)",3);
		
	}else{
		LcdSetCursorXY(6,1);
		USART_Transmit_lcd_string("( )",3);
		LcdSetCursorXY(11,1);
		USART_Transmit_lcd_string("( )",3);
	}
	if(signals_lights.low_lights_on){
		LcdSetCursorXY(3,0);
		USART_Transmit_lcd_string("(*)",3);
		LcdSetCursorXY(14,0);
		USART_Transmit_lcd_string("(*)",3);
	}else{
		LcdSetCursorXY(3,0);
		USART_Transmit_lcd_string("( )",3);
		LcdSetCursorXY(14,0);
		USART_Transmit_lcd_string("( )",3);
	}
	
	
	//set cruise_speed with encoder wheel

	if(input==-1){
		if(cruise_control.set_speed > 0){
			cruise_control.set_speed--;
		}		
	
	}else if(input==1){
		if(cruise_control.set_speed < cruise_control.cruise_max_speed){
			cruise_control.set_speed++;
		}	
	}
	else if(input<-1){
		if(cruise_control.set_speed > 9){
			cruise_control.set_speed -= 10;
		}
	
	}else if(input>1){
		if(cruise_control.set_speed < cruise_control.cruise_max_speed-9){
			cruise_control.set_speed += 10;
		}
	
	}

	
	char out_string[20];

	
	if(cruise_control.cruise_active){
		if(cruise_control.safety_off_active){
			sprintf(out_string, "Cruise: %3d On      ", cruise_control.set_speed);
		}else{
			sprintf(out_string, "Cruise: %3d On Soff!", cruise_control.set_speed);
		
		}
	}else{
		sprintf(out_string, "Cruise: %3d Off     ", cruise_control.set_speed);
	}
		LcdSetCursorXY(0,2);
		USART_Transmit_lcd_string(out_string,19);

	LcdSetCursorXY(0,3);
	sprintf(out_string, "Pedal : %3d  ", signals_lights.pedal_position);
	USART_Transmit_lcd_string(out_string,19);

	return 0;
}

int DrawStatus(int input){
	if(new_menu_refresh){
		clear_display();
		new_menu_refresh = 0;
	}

	char out_string[20];


	int i;


	sprintf(out_string, "Vbat %2.1fV", ((double)(board_status.input_voltage))*0.0355);
	
	lcd_set_cursor(0);
	USART_Transmit_lcd_string(out_string,20);

	//print batterybar
	lcd_set_cursor(10);
	USART_Transmit_lcd(0x5B);
	lcd_set_cursor(19);
	USART_Transmit_lcd(0x5D);

	int blocknumber = ((board_status.input_voltage)-619)/11;
	if(blocknumber<0){
		blocknumber = 0;
	}
	if(blocknumber>8){
		blocknumber = 8; 
	}
	lcd_set_cursor(11);
	for(i = 0;i<8;i++){
		lcd_set_cursor(11+i);
		if(i<blocknumber){
			USART_Transmit_lcd(0xFF);
		}
		else{
			USART_Transmit_lcd(0x20);
		}
	}


	//print module-status
	LcdSetCursorXY(0,2);
	USART_Transmit_lcd_string("FC",2);
	LcdSetCursorXY(4,2);
	USART_Transmit_lcd_string("MT",2);
	LcdSetCursorXY(8,2);
	USART_Transmit_lcd_string("FL",2);
	LcdSetCursorXY(12,2);
	USART_Transmit_lcd_string("TM",2);


	LcdSetCursorXY(0,3);
	if(module_status.fuel_cell == 0x0c){
		USART_Transmit_lcd_string("ok ",3);
	}else{
		USART_Transmit_lcd_string("err",3);
	}
	LcdSetCursorXY(4,3);
	if(module_status.back_light == 0x0c){
		USART_Transmit_lcd_string("ok ",3);
	}else{
		USART_Transmit_lcd_string("err",3);
	}
	LcdSetCursorXY(8,3);
	if(module_status.front_light == 0x0c){
		USART_Transmit_lcd_string("ok ",3);
	}else{
		USART_Transmit_lcd_string("err",3);
	}
	LcdSetCursorXY(12,3);
	if(module_status.telemetry == 0x0c){
		USART_Transmit_lcd_string("ok ",3);
	}else{
		USART_Transmit_lcd_string("err",3);
	}

	


	return 0;
}
int DrawFuelCellStatus(int input){
	if(new_menu_refresh){
		clear_display();
		new_menu_refresh = 0;
	}

	char out_string[30];
	sprintf(out_string, "P1 %1.2f P2 %1.2f ", ( ((double) fuelcell.pressure_sensor[0]) * 11.0) / 2024.0,( ((double) fuelcell.pressure_sensor[1]) * 11.0) / 2024.0);
	
	LcdSetCursorXY(0,0);
	USART_Transmit_lcd_string(out_string,20);
	
	sprintf(out_string, "Voltage: %2.2fV   ", ((double)fuelcell.pressure_sensor[2]*37)/512);
	
	LcdSetCursorXY(0,1);
	USART_Transmit_lcd_string(out_string,20);
	
	

	return 0;
}
int DrawMotorStatus(int input){
	if(new_menu_refresh){
		clear_display();
		new_menu_refresh = 0;
	}
	int i;
	lcd_set_cursor(0);
	for(i = 160;i<240;i++){
		
			USART_Transmit_lcd(i);
	}
	return 0;
}

int DrawGpsStatus(int input){
	if(new_menu_refresh){
		clear_display();
		new_menu_refresh = 0;
	}

	char out_string[20];

	sprintf(out_string, "%02d:%02d:%02d  %02d.%02d.%02d",rtc_struct.Hours, rtc_struct.Minutes, rtc_struct.Seconds, rtc_struct.Day, rtc_struct.Month, rtc_struct.Year);
		
	LcdSetCursorXY(1,3);
	USART_Transmit_lcd_string(out_string,20);
	

	return 0;
}



int DrawLog(int input){
	if(new_menu_refresh){
		clear_display();
		new_menu_refresh = 0;
	}
	int i;
	lcd_set_cursor(0);
	for(i = 0;i<80;i++){
		USART_Transmit_lcd(i);
	
	}

	return 0;
}
int DrawLightStatus(int input){
	if(new_menu_refresh){
		clear_display();
		new_menu_refresh = 0;
	}
	int i;
	lcd_set_cursor(0);
	for(i = 240;i<256;i++){
		if(i!=254){
			USART_Transmit_lcd(i);
		}
	}
	return 0;
}

int DrawMatrix(int input){

	static char counter = 0;
	
	static char lock_array[20] = {0};
	
	static unsigned char start_pos[20] = {0};
	static unsigned char stop_pos[20] = {0};



	counter++;

	if(counter>7){
		counter = 0;
		int i;
		for(i = 0;i<20;i++){
			
			if(lock_array[i] == 0 && (rand() / (RAND_MAX / 4 + 1) > 2)){//line open
				
				start_pos[i] = 0;
				stop_pos[i] = start_pos[i];
				DrawSymbolXY(i, start_pos[i], rand() / (RAND_MAX / 75 + 1) + 157);
				stop_pos[i]++;

				lock_array[i] = 1;
				
			}
			else{
				if(stop_pos[i]>3){//line full, start removing
					DrawSymbolXY(i, start_pos[i], 0x20);
					start_pos[i]++;
					if(start_pos[i]>3){
						lock_array[i] = 0;
					
					}
				}
				else{
					DrawSymbolXY(i, stop_pos[i], rand() / (RAND_MAX / 79 + 1) + 160);
					stop_pos[i]++;
				}
			}

		}
	}

	return 0;
}



//input: positions encoder wheel moved since last time DrawMainMenu was called
//returns: current pointer position
int DrawMainMenu(int cursor_movement){

	static int cursor_position = 0;
	static int last_cursor_position = 0;
	static int top_position = 0;
	static unsigned char redraw_needed = 1;
	static unsigned char redraw_cursor_needed = 1;


	if(new_menu_refresh){
		redraw_needed = 1;
		redraw_cursor_needed = 1;
		new_menu_refresh = 0;
	}

	int i,j;
	//change cursor and top position according to pointer_movement and pointer_position
	//find new cursor position
	cursor_movement = cursor_movement%MAIN_MENU_LENGTH;

	if(cursor_movement<0){
		cursor_position += cursor_movement + MAIN_MENU_LENGTH;

	}
	else{
		cursor_position += cursor_movement;
		
	}
	cursor_position = cursor_position%MAIN_MENU_LENGTH;

	if(cursor_position != last_cursor_position){
		redraw_cursor_needed = 1;
		last_cursor_position = cursor_position;
	}



	//update cursor position in menutable

	for(i = 0; i<MAIN_MENU_LENGTH;i++){
		
		if(i==cursor_position){
			main_menu_elements[i].show_cursor = 1;
		
		}
		else{
			main_menu_elements[i].show_cursor = 0;
		
		}
	
	
	}

	//update top_position if cursor is outside window
	if(cursor_position>top_position+3){
		top_position = cursor_position-3;
		if(top_position>MAIN_MENU_LENGTH-4){
			top_position = MAIN_MENU_LENGTH-4;
		}
		redraw_needed = 1;
	}
	else if(cursor_position<top_position){
		top_position = cursor_position;
		
		redraw_needed = 1;
	}

	
	//draw text
	if(redraw_needed){
		clear_display();
		
		j = 0;
		for(i = top_position;i<(top_position+4);i++){
			LcdSetCursorXY(1,j);
			USART_Transmit_lcd_progmem_string(main_menu_elements[i].pMenuText,18);
			j++;
		}
		redraw_needed = 0;
	}
	
	//draw cursor
	if(redraw_cursor_needed){
		clear_cursors();
		j = 0;
		for(i = top_position;i<(top_position+4);i++){
		
			if(main_menu_elements[i].show_cursor){
				DrawSymbolXY(0, j, 0x7E);
			}
			j++;
		}
		redraw_cursor_needed = 0;
	}

	return cursor_position;
}

int DrawSetupMenu(int cursor_movement, unsigned char element_selected, SETUP_MENU_ELEMENTS* menu_elements, unsigned char menu_length, DRAW_MENU_STATUS* draw_menu_status){


	if(new_menu_refresh){
		draw_menu_status->redraw_needed = 1;
		draw_menu_status->redraw_cursor_needed = 1;
		draw_menu_status->redraw_value_needed = 1;
		draw_menu_status->cursor_position = 0;
		draw_menu_status->top_position = 0;
		new_menu_refresh = 0;
	}
	if(select_refresh){
	
		draw_menu_status->redraw_cursor_needed = 1;
		select_refresh = 0;
	}
	int i,j;

	if(element_selected){ //change the current menu value instead of moving cursor
		if(cursor_movement < 2 && cursor_movement > -2){	
			menu_elements[draw_menu_status->cursor_position].value += cursor_movement;
		}
		else{//move more if encoder is turned fast
			menu_elements[draw_menu_status->cursor_position].value += cursor_movement*10;
		}
		if(menu_elements[draw_menu_status->cursor_position].value > menu_elements[draw_menu_status->cursor_position].value_max){
			menu_elements[draw_menu_status->cursor_position].value = menu_elements[draw_menu_status->cursor_position].value_max;
		}
		else if(menu_elements[draw_menu_status->cursor_position].value < menu_elements[draw_menu_status->cursor_position].value_min){
			menu_elements[draw_menu_status->cursor_position].value = menu_elements[draw_menu_status->cursor_position].value_min;
		}

		if(cursor_movement != 0){
			
			menu_elements[draw_menu_status->cursor_position].value_changed = 1;
			draw_menu_status->redraw_value_needed = 1;
		}
	}
	else{
	//change cursor and top position according to pointer_movement and pointer_position
	//find new cursor position


		cursor_movement = cursor_movement%menu_length;

		if(cursor_movement<0){
			draw_menu_status->cursor_position += cursor_movement + menu_length;

		}
		else{
			draw_menu_status->cursor_position += cursor_movement;
		
		}
		draw_menu_status->cursor_position = draw_menu_status->cursor_position%menu_length;

		if(draw_menu_status->cursor_position != draw_menu_status->last_cursor_position){
			draw_menu_status->redraw_cursor_needed = 1;
			draw_menu_status->last_cursor_position = draw_menu_status->cursor_position;
		}

	}

	//update cursor position in menutable

	for(i = 0; i<menu_length;i++){
	
		if(i==draw_menu_status->cursor_position){
			if(element_selected){
				menu_elements[i].show_cursor = 2;
			}else{
				menu_elements[i].show_cursor = 1;
			}
		}
		else{
			menu_elements[i].show_cursor = 0;
	
		}


	}
	
	//update top_position if cursor is outside window
	if(draw_menu_status->cursor_position>draw_menu_status->top_position+3){
		draw_menu_status->top_position = draw_menu_status->cursor_position-3;
		if(draw_menu_status->top_position>menu_length-4){
			draw_menu_status->top_position = menu_length-4;
		}
		draw_menu_status->redraw_needed = 1;
		draw_menu_status->redraw_value_needed = 1;
	}
	else if(draw_menu_status->cursor_position<draw_menu_status->top_position){
		draw_menu_status->top_position = draw_menu_status->cursor_position;
		
		draw_menu_status->redraw_needed = 1;
		draw_menu_status->redraw_value_needed = 1;
	}

	
	//draw text
	if(draw_menu_status->redraw_needed){
		clear_display();
		
		j = 0;
		unsigned char length;
		if(menu_length<4){
			length = menu_length;
		}else{
			length = 4;
		}

		for(i = draw_menu_status->top_position;i<(draw_menu_status->top_position+length);i++){
		
			LcdSetCursorXY(1,j);
			USART_Transmit_lcd_progmem_string(menu_elements[i].pMenuText,18);
			j++;
		}
		draw_menu_status->redraw_needed = 0;
	}
	
	//draw values
	if(draw_menu_status->redraw_value_needed){
		clear_values();
		j = 0;
		char string[4];
		unsigned char length;
		if(menu_length<4){
			length = menu_length;
		}else{
			length = 4;
		}
		for(i = draw_menu_status->top_position;i<(draw_menu_status->top_position+length);i++){
			if(menu_elements[i].value != -1){
				LcdSetCursorXY(17,j);
				//check if only 0 and 1 is valid choices, then write on or off instead
				if(menu_elements[i].value_min == 0 && menu_elements[i].value_max == 1){
					if(menu_elements[i].value){
						sprintf(string,"on ");
						
					}else{
						sprintf(string,"off");
					}
				}
				else{
					sprintf(string,"%d",menu_elements[i].value);
				}

				USART_Transmit_lcd_string(string,4);
			}
			j++;
			
		
		}
		draw_menu_status->redraw_value_needed = 0;
	}

	//draw cursor
	if(draw_menu_status->redraw_cursor_needed){
		clear_cursors();
		j = 0;
		for(i = draw_menu_status->top_position;i<(draw_menu_status->top_position+4);i++){
		
			if(menu_elements[i].show_cursor == 1){
				LcdSetCursorXY(0,j);

				USART_Transmit_lcd(0x7E);
		
			}
			else if(menu_elements[i].show_cursor == 2){
			
				switch(j){
					case 0:
						lcd_set_cursor(16);
					break;
					case 1:
						lcd_set_cursor(80);
					break;
					case 2:
						lcd_set_cursor(36);
					break;
					case 3:
						lcd_set_cursor(100);
					break;		
				}

				USART_Transmit_lcd(0x7E);
			
			
			}
			j++;
		}
		draw_menu_status->redraw_cursor_needed = 0;
	}

	return draw_menu_status->cursor_position;


}


void clear_cursors(void){
	
	lcd_set_cursor(0);
	USART_Transmit_lcd(0x20);
	lcd_set_cursor(64);
	USART_Transmit_lcd(0x20);
	lcd_set_cursor(20);
	USART_Transmit_lcd(0x20);
	lcd_set_cursor(84);
	USART_Transmit_lcd(0x20);
	
	lcd_set_cursor(16);
	USART_Transmit_lcd(0x20);
	lcd_set_cursor(80);
	USART_Transmit_lcd(0x20);
	lcd_set_cursor(36);
	USART_Transmit_lcd(0x20);
	lcd_set_cursor(100);
	USART_Transmit_lcd(0x20);

}

void clear_values(void){
	lcd_set_cursor(17);
	USART_Transmit_lcd(0x20);
	USART_Transmit_lcd(0x20);
	USART_Transmit_lcd(0x20);
	
	lcd_set_cursor(81);
	USART_Transmit_lcd(0x20);
	USART_Transmit_lcd(0x20);
	USART_Transmit_lcd(0x20);

	lcd_set_cursor(37);
	USART_Transmit_lcd(0x20);
	USART_Transmit_lcd(0x20);
	USART_Transmit_lcd(0x20);

	lcd_set_cursor(101);
	USART_Transmit_lcd(0x20);
	USART_Transmit_lcd(0x20);
	USART_Transmit_lcd(0x20);

}
void MenuSetDefaultValues(void){

	HandleOptionMenuChanges();
	HandleLightsMenuChanges();
}












