
#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "macro.h"
#include "timer0.h"


TIMER_CALLBACK_FUNC CallbackFunc[TIMER0_NUM_CALLBACKS];


unsigned char CallbackInterval[TIMER0_NUM_CALLBACKS];
unsigned char CallbackCounter[TIMER0_NUM_CALLBACKS];

void Timer0Init(void){

	unsigned char i;

    // Initialize array of callback functions
    for (i=0; i<TIMER0_NUM_CALLBACKS; i++){
        CallbackFunc[i] = NULL;
		CallbackInterval[i] = 255;
		CallbackCounter[i] = 255;
	}
    // Initialize 8bit Timer0.
	
	TCCR0A = (1 << CS02);

	ENABLE_TM0OVF_INT;
}

//interval: interrupt is called 122 times per second, interval = 122 means callback happens 1 time per second
unsigned char Timer0RegisterCallback(TIMER_CALLBACK_FUNC pFunc, unsigned char interval){
	
	unsigned char i;
    
	//return true if already registered
    for (i=0; i<TIMER0_NUM_CALLBACKS; i++){
        if (CallbackFunc[i] == pFunc)
            return TRUE;
    }
    
    for (i=0; i<TIMER0_NUM_CALLBACKS; i++)
    {
        if (CallbackFunc[i] == NULL)
        {
            CallbackFunc[i] = pFunc;
			CallbackInterval[i] = interval;
			CallbackCounter[i] = 1;
            return TRUE;
        }   
    }
    
    return FALSE;
}
unsigned char Timer0RemoveCallback(TIMER_CALLBACK_FUNC pFunc){

   	unsigned char i;
    
    for (i=0; i<TIMER0_NUM_CALLBACKS; i++)
    {
        if (CallbackFunc[i] == pFunc)
        {
            CallbackFunc[i] = NULL;
			CallbackInterval[i] = 255;
            return TRUE;
        }
    }
        
    return FALSE;
}


ISR(TIMER0_OVF_vect){

   	unsigned char i;
    
    for (i=0; i<TIMER0_NUM_CALLBACKS; i++){
		if (CallbackFunc[i] != NULL){
			CallbackCounter[i]--;
			if(CallbackCounter[i] == 0){
				CallbackCounter[i] = CallbackInterval[i];
				CallbackFunc[i]();
			}
		}            
    }
}
