/* rev 1.0 2/2010			written by Anders Guldahl			     */
#include <stdio.h>

#include <inttypes.h>
#include <avr/io.h>
/*************************************************************************
* description:  uartdriver for avr 128
**************************************************************************/
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "macro.h"
#include "uart_lcd.h"


void USART_Init_lcd(void){
	unsigned int brr;


	brr = 103;

	/* Set baud rate */
	UBRR1H = (unsigned char)(brr>>8);
	UBRR1L = (unsigned char)brr;

	UCSR1A |= (1<<U2X1);
	/* Enable transmitter */
	UCSR1B = (1<<TXEN1);

	/* Set frame format: 8data, 1stop bit */
	UCSR1C = (3<<UCSZ10);

/*
	stdout = uart;
	fdevopen (&USART_Transmit_debug, &USART_Receive_debug);
*/


}

void USART_Transmit_lcd( unsigned char data){
	/* Wait for empty transmit buffer */
	while ( !( UCSR1A & (1<<UDRE1)) );

	/* Put data into buffer, sends the data */
	UDR1 = data;

}

void USART_Transmit_lcd_string(char * string, unsigned char length){
	
	int i;
	for(i = 0; i<length;i++){
		
		if(string[i]=='\0'){
			break;
		}
		USART_Transmit_lcd(string[i]);
	}


}

void USART_Transmit_lcd_progmem_string(const char *pFlashStr, unsigned char length){
	
	int i;
	
	for(i = 0; i<length;i++){
		
		if(pgm_read_byte(&pFlashStr[i])=='\0'){
			break;
		}
		USART_Transmit_lcd(pgm_read_byte(&pFlashStr[i]));
	}


}


void set_lcd_brightness(int brightness){
	
	
	if(brightness<30 && brightness>=0){
		USART_Transmit_lcd(0x7C);
	
		USART_Transmit_lcd(128+brightness);
	}
	
}

void clear_display(void){
	
	USART_Transmit_lcd(0xFE);
	USART_Transmit_lcd(0x01);

}

void lcd_set_cursor(unsigned char position){
	if(position<40 || (position>63 && position<104)){
		USART_Transmit_lcd(0xFE);
		USART_Transmit_lcd(128+position);
	}

	

}

void LcdSetCursorXY(char x, char y){

	if(y<4 && y>=0 && x<20 && x>=0){
		switch(y){
			case 0:
				lcd_set_cursor(x);
			break;
			case 1:
				lcd_set_cursor(x + 64);
			break;
			case 2:
				lcd_set_cursor(x + 20);
			break;
			case 3:
				lcd_set_cursor(x + 84);
			break;	
		}
	}
}

void DrawSymbolXY(char x, char y, char symbol){

	LcdSetCursorXY(x, y);
	USART_Transmit_lcd(symbol);
}

void lcd_blinking_box(char on){
		USART_Transmit_lcd(0xFE);
	if(on){
		USART_Transmit_lcd(0x0D);
	}
	else{
		USART_Transmit_lcd(0x0C);
	
	}

}

void init_spark_lcd(void){
	
	USART_Transmit_lcd(0x7c);
	_DELAY_MS(10);
	USART_Transmit_lcd(0x10);
	_DELAY_MS(10);
	unsigned int brr;


	brr = 25;

	/* Set baud rate */
	UBRR1H = (unsigned char)(brr>>8);
	UBRR1L = (unsigned char)brr;

	UCSR1A |= (1<<U2X1);
	/* Enable transmitter */
	UCSR1B = (1<<TXEN1);

	/* Set frame format: 8data, 1stop bit */
	UCSR1C = (3<<UCSZ10);


}


