/* rev 1.0 01/2010 				written by Anders Guldahl			     */

#include <stdio.h>
#include <string.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

#include "macro.h"
#include "main.h"
#include "uart_debug.h"
#include "uart_BT.h"
#include "status_led.h"
#include "voltage_adc.h"
#include "can.h"
#include "SPI_routines.h"
#include "mcp23S17.h"
#include "led_driver.h"
#include "data_structs.h"
#include "../common_files/eeprom.h"
#include "../common_files/status_structs.h"





BOARD_STATUS board_status;

SENSOR_STATUS sensor_status;



//initialize module
unsigned char init(void){

	board_status.reset_source = MCUSR;
	MCUSR = 0;
	wdt_disable();

	eepromResetBootByte();
	add_to_PowerOnCount();


	USART_Init_debug();
	USART_Init_BT();

	init_can();
	init_adc();
	spi_init();
	init_status_led();
	init_led_driver();
	init_io_expander();

	//init horn out on PF0
	PORTF &= ~(1<<PF0);
	DDRF |= (1<<PF0);


	heart_beat(1);
	set_status_led(GREEN);



	sei();
	return 0;
}





int main(){


	init();


	set_status_led(GREEN);


	char output_string[200];
	char *lcd_string = "hello this is a test";
	unsigned char data;
	int counter = 0;

	union can_frame send;
	union can_frame msg;
	union can_frame *recv;
	recv = &msg;

	int i;
	unsigned char state = 100;
	unsigned char gray_pedal_position = 0;
	unsigned char io_registers[22] = {0};
	unsigned char lcd_ready = 0;
	unsigned char BT_light_ctrl = 0;
	
	unsigned int last_system_ticks = 0;

	printf("Times powered on: %d reset_source %x\n\r", board_status.times_powered_on, board_status.reset_source);

	

	while(1){

		counter ++;

		board_status.mcu_voltage = read_mcu_voltage();
		board_status.board_voltage = read_board_voltage();
		board_status.input_voltage = read_input_voltage();

		
		sprintf(output_string,"%d time: %d\n\r",board_status.input_voltage, get_system_ticks());

		if(last_system_ticks != get_system_ticks()){
			//USART_Transmit_BT_string(output_string,30);
			last_system_ticks = get_system_ticks();
		}

		//printf("counter: %d mcu %d,board %d,input %d\r\n",counter,status.mcu_voltage,status.board_voltage,status.input_voltage);

		
		unsigned char new_pedal_position = get_pedal_position();

		if(sensor_status.pedal_position != new_pedal_position){
			sensor_status.pedal_position = new_pedal_position;
			sendCANmsgPedalPosition();

		}
		
		
		


		//quick hack uart-debug commands, mostly changing LED-color, useful for testpurposes

		data = USART_Receive_debug();

			switch (data){
			case'1':
				if(OCR3AL>0){
					OCR3AL--;
				}

			break;
			case'2':
				if(OCR3AL<0xff){
					OCR3AL++;
				}
			break;
			case'3':
				if(OCR3BL>0){
					OCR3BL--;
				}
			break;
			case'4':
				if(OCR3BL<0xff){
					OCR3BL++;
				}
			break;
			case'5':
				if(OCR3CL>0){
					OCR3CL--;
				}
			break;
			case'6':
				if(OCR3CL<0xff){
					OCR3CL++;
				}
			break;
			case'7':
				PORTB &= ~(1<<PB4);
			
			break;
			case'8':
				PORTB |= (1<<PB4);
			
			break;
			case'9':

			break;
			case'q':

			break;
			case'w':
				set_status_led(LED_OFF);
			break;
			case't':
			

			break;
			case'r':
				wdt_enable(WDTO_15MS);
				while(1); //wait for watchdog

			break;
	
		}

		send.id = 0x745;			
		send.length = 5;
		send.rbr = 0;
		send.rb0 = 0;
		send.data[0] = 0;
		send.data[1] = 0;
		send.data[2] = 0;
		send.data[3] = 0;
		send.data[4] = 0;
		

		data = USART_Receive_BT();
		switch (data){
			//Low beam
			case'a':
				send.data[0] = 1;
				send_can_frame(&send);
			break;
	
			//high beam
			case'c':
				send.data[1] = 1;
				send_can_frame(&send);
			break;

			//emergency
			case'e':
				send.data[2] = 1;
				send_can_frame(&send);
			break;

			//showmode
			case'g':
				send.data[3] = 1;
				send_can_frame(&send);
			break;

			//horn
			case'i':
				send.data[4] = 1; 
				send_can_frame(&send);
			break;

		}
		
		
		

		//send and receive CAN-messages

	
		while(new_can_frame()){//receive all new can-frames
			recv = read_can_frame();
			next_can_frame();

			//reset module if requested from CAN-message for bootloaderapplication

			if(recv->id == 0x5E0){
				switch (recv->data[0]){
					case 1:
						OCR3AL = 0x00;
					break;
					case 2:
						OCR3AL = 0xff;
					break;
					case 3:
						fast_breathing(1);
					break;
					case 4:
						set_status_led(RED);
					break;
					case 5:
						set_status_led(GREEN);
					break;
					case 6:
						set_status_led(BLUE);
					break;
					case 7:
						set_status_led(YELLOW);
					break;
					case 8:
						set_status_led(WHITE);
					break;	
					case 9:
						set_status_led(MAGENTA);
					break;									
					case 10:
						set_status_led(CYAN);
					break;
					case 11:
						set_status_led(ORANGE);
					break;
					case 12:
						set_status_led(LED_OFF);
					break;

				}
			
			}
			//lights
			else if(recv->id == 0x5C1){
				if(!BT_light_ctrl){
					OCR3AL = recv->data[2];
					OCR3BL = recv->data[1];
					OCR3CL = recv->data[0];
					OCR2A = recv->data[3]; 
				}
			}

		}


		//send msg over CAN
		SendStatusMsg(0x0C);
		

		_DELAY_MS(10);
	}
}


void sendCANmsgPedalPosition(void){
	union can_frame send;
	send.id = 0x742;			
	send.length = 1;
	send.rbr = 0;
	send.rb0 = 0;

		
	send.data[0] = sensor_status.pedal_position;


	send_can_frame(&send);

}


void SendStatusMsg(unsigned char status){
		
	static unsigned int last_system_ticks = 0;
	if(last_system_ticks != get_system_ticks()){
		union can_frame send;
		send.id = 0x7C8;			
		send.length = 1;
		send.rbr = 0;
		send.rb0 = 0;
		send.data[0] = status;
		send_can_frame(&send);

		//status to dashboard
		send.id = 0x754;
		send.length = 8;
		send.rbr = 0;
		send.rb0 = 0;
		send.data[0] = status;
		int i;
		for(i=1;i<8;i++){
			send.data[i] = 0;
		}
		send_can_frame(&send);		

		last_system_ticks = get_system_ticks();
	}
	
}
