
/* rev 1.0 02/2010 				written by Anders Guldahl			     */

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "macro.h"
#include "SPI_routines.h"
#include "mcp23S17.h"
#include "can.h"

const prog_uint8_t ace_tab[] = {0,56,40,55,
24,0,39,52,8,57,0,0,23,0,36,13,120,0,41,54,
0,0,0,53,7,0,0,0,20,19,125,18,104,105,0,0,
25,106,38,0,0,58,0,0,0,0,37,14,119,118,0,0,
0,107,0,0,4,0,3,0,109,108,2,1,88,0,89,0,
0,0,0,51,9,10,90,0,22,11,0,12,0,0,42,43,
0,0,0,0,0,0,0,0,21,0,126,127,103,0,102,0,
0,0,0,0,0,0,91,0,0,0,0,0,116,117,0,0,115,
0,0,0,93,94,92,0,114,95,113,0,72,71,0,68,73,
0,0,29,0,70,0,69,0,0,35,34,121,0,122,0,74,0,
0,30,6,0,123,0,0,0,124,17,0,0,0,67,26,0,
27,28,0,59,0,0,0,0,0,15,0,0,0,0,0,0,0,
0,5,0,0,0,110,0,111,16,87,84,0,45,86,85,
0,50,0,0,0,46,0,0,0,33,0,83,0,44,75,0,
0,31,0,0,0,0,0,0,0,32,100,61,101,
66,0,62,0,49,99,60,0,47,0,0,0,48,77,82,78,
65,76,63,0,64,98,81,79,80,97,96,112
};

volatile unsigned char rotations = 0;

unsigned char init_io_expander(void){

	
	SD_CS_ASSERT;

	SPI_transmit(0x40);	//write mode
	SPI_transmit(0x00); //start at register 0x00

	SPI_transmit(0xff); //reg:0x00 Port A input
	SPI_transmit(0x03); //reg:0x01 pin 0 and 1 input, 2-7 output port B
	SPI_transmit(0x00); //reg:0x02 polarityA
	SPI_transmit(0x00); //reg:0x03 polarityB
	SPI_transmit(0x00); //reg:0x04 INTA 
	SPI_transmit(0x02); //reg:0x05 INT B enabled on pin 2
	SPI_transmit(0x00); //reg:0x06 DEFVALA
	SPI_transmit(0x00); //reg:0x07 DEFVALB
	SPI_transmit(0x00); //reg:0x08 INTCONA //interrupt on pin-change is active 0
	SPI_transmit(0x00); //reg:0x09 INTCONB
	SPI_transmit(0x04); //reg:0x0A IOCON	
	SPI_transmit(0x04); //reg:0x0B IOCON	//same as above?
	SPI_transmit(0x00); //reg:0x0C PULLUPA
	SPI_transmit(0x00); //reg:0x0D PULLUPB
	SPI_transmit(0x00); //reg:0x0E INTFLAG A
	SPI_transmit(0x00); //reg:0x0F INTFLAG B
	SPI_transmit(0x00); //reg:0x10 INT CAPTURE A
	SPI_transmit(0x00); //reg:0x11 INT CAPTURE B
	SPI_transmit(0x00); //reg:0x12 GPIOA
	SPI_transmit(0x00); //reg:0x13 GPIOB
	SPI_transmit(0x00); //reg:0x14 OUTPUT LATCH A
	SPI_transmit(0x00); //reg:0x15 OUTPUT LATCH B
	
		
	SD_CS_DEASSERT;


	
	
	//init io_B interruptpin on avr pin int0, enable pull-up, trigger on falling edge
	DDRD &= ~(1<<PD0); //set as input
	PORTD |= (1<<PD0); //enable pullup

	EICRA &= ~((1<<ISC01)|(1<<ISC00)); //low level interrupt
	
	ENABLE_EXT_INT0; //enable interrupt on pin1




	return 0;
}


unsigned char set_io_bit(char port, unsigned char pin){
	
	DISABLE_EXT_INT0;
	if(port == 'A'){
		SD_CS_ASSERT;
		SPI_transmit(0x41);	//read mode
		SPI_transmit(0x14); //register 0x14 OLATA
		unsigned char port = SPI_receive();
		SD_CS_DEASSERT;

		port |= (1<<pin); // set bit
		
		//write back
		SD_CS_ASSERT;
		SPI_transmit(0x40);	//write mode
		SPI_transmit(0x14); //register 0x14 OLATA
		SPI_transmit(port);
		SD_CS_DEASSERT;

	
	}
	else if(port == 'B'){
		SD_CS_ASSERT;
		SPI_transmit(0x41);	//read mode
		SPI_transmit(0x15); //register 0x15 OLATB
		unsigned char port = SPI_receive();
		SD_CS_DEASSERT;

		port |= (1<<pin); // set bit
		
		//write back
		SD_CS_ASSERT;
		SPI_transmit(0x40);	//write mode
		SPI_transmit(0x15); //register 0x15 OLATB
		SPI_transmit(port);
		SD_CS_DEASSERT;


	}
	ENABLE_EXT_INT0;
	return 0;
}

unsigned char clear_io_bit(char port, unsigned char pin){

	DISABLE_EXT_INT0;
	if(port == 'A'){
		SD_CS_ASSERT;
		SPI_transmit(0x41);	//read mode
		SPI_transmit(0x14); //register 0x14 OLATA
		unsigned char port = SPI_receive();
		SD_CS_DEASSERT;

		port &= ~(1<<pin); // clear bit
		
		//write back
		SD_CS_ASSERT;
		SPI_transmit(0x40);	//write mode
		SPI_transmit(0x14); //register 0x14 OLATA
		SPI_transmit(port);
		SD_CS_DEASSERT;

	
	}
	else if(port == 'B'){
		SD_CS_ASSERT;
		SPI_transmit(0x41);	//read mode
		SPI_transmit(0x15); //register 0x15 OLATB
		unsigned char port = SPI_receive();
		SD_CS_DEASSERT;

		port &= ~(1<<pin); // clear bit
		
		//write back
		SD_CS_ASSERT;
		SPI_transmit(0x40);	//write mode
		SPI_transmit(0x15); //register 0x15 OLATB
		SPI_transmit(port);
		SD_CS_DEASSERT;


	}
	ENABLE_EXT_INT0;
	return 0;
}

void write_led_gpio_byte(unsigned char byte){

	
	byte &= 0x3f;
	byte = byte<<2;
	
	DISABLE_EXT_INT0;
	SD_CS_ASSERT;
	SPI_transmit(0x40);	//write mode
	SPI_transmit(0x15); //register 0x15 OLATB
	SPI_transmit(byte);
	SD_CS_DEASSERT;

	ENABLE_EXT_INT0;
}

unsigned char get_pedal_position(void){
	DISABLE_EXT_INT0;

	SD_CS_ASSERT;
	SPI_transmit(0x41);	//read mode
	SPI_transmit(0x12); //register 0x12 GPIOA
	unsigned char pedal_pos = SPI_receive();
	SD_CS_DEASSERT;

	ENABLE_EXT_INT0;
	return pgm_read_byte(&ace_tab[pedal_pos]);


}

unsigned char get_encoder_rotations(void){
	return rotations;
}


volatile unsigned char state = 0;
volatile signed char state_counter = 0;


ISR(INT0_vect){
	
	SD_CS_ASSERT;
	//read INTCAPB
	SPI_transmit(0x41);	//read mode
	SPI_transmit(0x11); //register intcapB 
	unsigned char io_reg = SPI_receive();
	SD_CS_DEASSERT;

	//brakepedal
	if(TEST_BIT(io_reg, 1)){ //released
			union can_frame send;
			send.id = 0x6C0;			
			send.length = 1;
			send.rbr = 0;
			send.rb0 = 0;
			send.data[0] = 0;
			send_can_frame(&send);

	}
	else{ //pushed
			union can_frame send;
			send.id = 0x6C0;			
			send.length = 1;
			send.rbr = 0;
			send.rb0 = 0;
			send.data[0] = 1;
			send_can_frame(&send);
			
	}



		

}
