/* rev 1.0 02/2010 				written by Anders Guldahl			     */

#include <avr/io.h>
#include "macro.h"
#include "SPI_routines.h"
#include "LT6802.h"




unsigned char init_battery_monitor(void){
	//init batterymonitor
	int i;
	SD_CS_ASSERT;

	SPI_transmit(0x01); //write command

	for(i = 0;i<4;i++){
		SPI_transmit(0x91); //levelpolling and dutycycle = 1
		SPI_transmit(0x00);
		SPI_transmit(0x00);
		SPI_transmit(0x00);
		SPI_transmit(0x00);
		SPI_transmit(0x00);
	}

	SD_CS_DEASSERT;
	
	
	return 0;
}
unsigned char read_cell_voltages(int *cell_voltage, unsigned char mode){
	
	unsigned char data;
	int i;
	SD_CS_ASSERT;

	SPI_transmit(mode);

	unsigned char counter = 0;
	//wait for conversion complete
	while(SPI_receive() == 0x00){
		counter++;
		if(counter>50){
			break;
		}
		_DELAY_MS(1);
	}
	

	SD_CS_DEASSERT;
	asm("nop");
	asm("nop");
	asm("nop");
	SD_CS_ASSERT;

	SPI_transmit(0x04);

	for(i = 0;i<3;i++){
		cell_voltage[i*12] = SPI_receive();
		data = SPI_receive();
		cell_voltage[i*12] |= ((data & 0x0F)<<8);

		cell_voltage[i*12+1] = data >> 4;
		cell_voltage[i*12+1] |= SPI_receive() << 4;

		cell_voltage[i*12+2] = SPI_receive();
		data = SPI_receive();
		cell_voltage[i*12+2] |= ((data & 0x0F)<<8);

		cell_voltage[i*12+3] = data >> 4;
		cell_voltage[i*12+3] |= SPI_receive() << 4;

		cell_voltage[i*12+4] = SPI_receive();
		data = SPI_receive();
		cell_voltage[i*12+4] |= ((data & 0x0F)<<8);

		cell_voltage[i*12+5] = data >> 4;
		cell_voltage[i*12+5] |= SPI_receive() << 4;

		cell_voltage[i*12+6] = SPI_receive();
		data = SPI_receive();
		cell_voltage[i*12+6] |= ((data & 0x0F)<<8);

		cell_voltage[i*12+7] = data >> 4;
		cell_voltage[i*12+7] |= SPI_receive() << 4;

		cell_voltage[i*12+8] = SPI_receive();
		data = SPI_receive();
		cell_voltage[i*12+8] |= ((data & 0x0F)<<8);

		cell_voltage[i*12+9] = data >> 4;
		cell_voltage[i*12+9] |= SPI_receive() << 4;

		cell_voltage[i*12+10] = SPI_receive();
		data = SPI_receive();
		cell_voltage[i*12+10] |= ((data & 0x0F)<<8);

		if(i == 2){
			SPI_receive(); //receive 0 voltage, but do not put in table
		}
		else{
			cell_voltage[i*12+11] = data >> 4;
			cell_voltage[i*12+11] |= SPI_receive() << 4;	
		}

		SPI_receive(); //PEC-byte
	}
	
	cell_voltage[35] = SPI_receive();
	data = SPI_receive();
	cell_voltage[35] |= ((data & 0x0F)<<8);

	cell_voltage[36] = data >> 4;
	cell_voltage[36] |= SPI_receive() << 4;

	cell_voltage[37] = SPI_receive();
	data = SPI_receive();
	cell_voltage[37] |= ((data & 0x0F)<<8);

	cell_voltage[38] = data >> 4;
	cell_voltage[38] |= SPI_receive() << 4;

	cell_voltage[39] = SPI_receive();
	data = SPI_receive();
	cell_voltage[39] |= ((data & 0x0F)<<8);

	cell_voltage[40] = data >> 4;
	cell_voltage[40] |= SPI_receive() << 4;

	cell_voltage[41] = SPI_receive();
	data = SPI_receive();
	cell_voltage[41] |= ((data & 0x0F)<<8);

	cell_voltage[42] = data >> 4;
	cell_voltage[42] |= SPI_receive() << 4;

	cell_voltage[43] = SPI_receive();
	data = SPI_receive();
	cell_voltage[43] |= ((data & 0x0F)<<8);

	cell_voltage[44] = data >> 4;
	cell_voltage[44] |= SPI_receive() << 4;

	cell_voltage[45] = SPI_receive();
	data = SPI_receive();
	cell_voltage[45] |= ((data & 0x0F)<<8);

	SPI_receive(); //receive 0 voltage, but do not put in table
	

	SPI_receive(); //PEC-byte




	SD_CS_DEASSERT;



	return 0;
}

unsigned char read_temperatures(int *temperature, unsigned char mode){
	
	unsigned char data;
	int i;
	SD_CS_ASSERT;

	SPI_transmit(mode);
	unsigned char counter = 0;
	//wait for conversion complete
	while(SPI_receive() == 0x00){
		counter++;
		if(counter>50){
			break;
		}
		_DELAY_MS(1);
	}
	

	SD_CS_DEASSERT;
	asm("nop");
	asm("nop");
	asm("nop");
	SD_CS_ASSERT;

	SPI_transmit(0x08);
	
	for(i = 0;i<4;i++){
		
		temperature[i*3] = SPI_receive();					
		data = SPI_receive();
		temperature[i*3] |= ((data & 0x0F)<<8);

		temperature[i*3+1] = data >> 4;					
		temperature[i*3+1] |= SPI_receive() << 4;

		temperature[i*3+2] = SPI_receive();
		data = SPI_receive();
		temperature[i*3+2] |= ((data & 0x0F)<<8);			
		
		SPI_receive(); //PEC byte
	}

	

	SD_CS_DEASSERT;


	return 0;
}
unsigned char battery_monitor_self_test(void){

	unsigned char result = 1;
	int i,j;
	
	SD_CS_ASSERT;

	SPI_transmit(0x1E);


	//wait for conversion complete
	unsigned char counter = 0;
	while(SPI_receive() == 0x00){
		counter++;
		if(counter>50){
			break;
		}
		_DELAY_MS(1);
	}
	
	SD_CS_DEASSERT;
	asm("nop");
	asm("nop");
	asm("nop");
	SD_CS_ASSERT;

	SPI_transmit(0x04);

	for(j = 0; j<4; j++){
		for(i = 0; i<18; i++){
			if(SPI_receive() != 0x55){
				result = 0;
		
			}
		}

		SPI_receive(); //PEC byte
	}

	SD_CS_DEASSERT;
	asm("nop");
	asm("nop");
	asm("nop");
	SD_CS_ASSERT;

	SPI_transmit(0x1F);

	
	//wait for conversion complete
	counter = 0;
	while(SPI_receive() == 0x00){
		counter++;
		if(counter>50){
			break;
		}
		_DELAY_MS(1);
	}
	
	SD_CS_DEASSERT;
	asm("nop");
	asm("nop");
	asm("nop");
	SD_CS_ASSERT;

	SPI_transmit(0x04);
	for(j = 0; j<4; j++){
		for(i = 0; i<18; i++){
			if(SPI_receive() != 0xAA){
				result = 0;
		
			}
		}

		SPI_receive(); //PEC byte
	}

	SD_CS_DEASSERT;


	return result;
}

unsigned char read_config_byte(unsigned char *config_byte){



	return 0;

}


