/* rev 1.0 01/2010 				written by Anders Guldahl			     */

#include <stdio.h>
#include <string.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

#include "macro.h"
#include "uart_debug.h"
#include "status_led.h"
#include "voltage_adc.h"
#include "can.h"
#include "SPI_routines.h"
#include "LT6802.h"
#include "main.h"
#include "temp_converting.h"
#include "can_fuelcell_msg.h"
#include "timer0.h"
#include "../common_files/eeprom.h"
#include "../common_files/status_structs.h"


BOARD_STATUS board_status;
FUELCELL fuelcell;



//initialize module
unsigned char init(void){

	board_status.reset_source = MCUSR;
	MCUSR = 0;
	wdt_disable();

	eepromResetBootByte();
	add_to_PowerOnCount();

	Timer0Init();
	USART_Init_debug();
	init_can();
	init_adc();
	spi_init();
	init_status_led();
	init_battery_monitor();
	

	//init io-lines
	PORTE &= ~((1<<PE3)|(1<<PE4)|(1<<PE5));
	DDRE |= (1<<PE3)|(1<<PE4)|(1<<PE5);

	PORTD &= ~((1<<PD2)|(1<<PD3));
	DDRD |= (1<<PD2)|(1<<PD3);
	
	

	heart_beat(1);
	set_status_led(GREEN);

	fuelcell.fan_dutycycle = 0;

	sei();
	return 0;
}





int main(){


	init();


	set_status_led(GREEN);


	char output_string[200];
	unsigned char data;
	int counter = 0;

	union can_frame send;
	union can_frame msg;
	union can_frame *recv;
	recv = &msg;


	printf("Times powered on: %d reset_source %x\n\r", board_status.times_powered_on, board_status.reset_source);



	


	int temperature[12] = {0};
	int cell_voltage[46] = {0};
	int current_sensor[3] = {0};
	unsigned char output[5] = {0};
	unsigned char config_byte[4];
	unsigned char purge = 0;	
	
	unsigned char turn_on = 0;
	unsigned char turn_off = 0;

	unsigned char measure_counter = 0;
	unsigned char interval = 1;


	int i,j;
	double value = 0;
	double voltage_sum = 0;
	unsigned char cell_undervoltage = 0;
	unsigned char cell_measure_mode = CELL_MODE_NORMAL;
	unsigned char temp_measure_mode = TEMP_MODE_NORMAL;
	unsigned int last_system_tick = 0;	

	while(1){

		//wait for new system_tick, to syncrhonize on seconds		

		counter ++;
		
		if(last_system_tick!=get_system_ticks()){
			last_system_tick = get_system_ticks();	
			measure_counter++;
		}
		
		/*
		status.mcu_voltage = read_mcu_voltage();
		status.board_voltage = read_board_voltage();
		status.input_voltage = read_input_voltage();
		//printf("counter: %d mcu %d,board %d,input %d\r\n",counter,board_status.mcu_voltage,board_status.board_voltage,board_status.input_voltage);
		*/
		
		read_cell_voltages(cell_voltage, cell_measure_mode);
		
		/*
		//undervoltage alarm
		for(i = 0; i<46;i++){

			if(cell_voltage[i]<130){
				send.id = 0x743;			
				send.length = 1;
				send.rbr = 0;
				send.rb0 = 0;
				send.data[0] = (unsigned char) i;

				send_can_frame(&send);
				

			}
		
		}
		*/

		read_temperatures(temperature, temp_measure_mode);
		for(i = 0; i<3;i++){
			current_sensor[i] = read_channel((unsigned char)i);
		}

		if(current_sensor[2]<483){
			SET_BIT(PORTD, PD2);
		}else{
			CLEAR_BIT(PORTD, PD2);
		}

		if(current_sensor[2] >483){
		
			CLEAR_BIT(PORTD, PD2);
		}

		output[0] = TEST_BIT(PORTE, PE3);
		output[1] = TEST_BIT(PORTD, PD3);
		output[2] = TEST_BIT(PORTD, PD2);
		output[3] = TEST_BIT(PORTE, PE4);
		output[4] = TEST_BIT(PORTE, PE5);

		//printf_routine
		if(measure_counter >= interval){

				
			USART_Transmit_debug('V');
			USART_Transmit_debug(',');
			voltage_sum = 0;
			for(i = 0; i<46;i++){
				
				value = ((double) cell_voltage[i])*0.0015;
				printf("%1.2f,",value);
			
				voltage_sum+=value;
			
			}

			USART_Transmit_debug('S');
			USART_Transmit_debug(',');
			printf("%1.2f,",((double)current_sensor[2]*37)/512);

			USART_Transmit_debug('T');
			USART_Transmit_debug(',');
			printTemperatures(temperature);					

			USART_Transmit_debug('P');
			USART_Transmit_debug(',');
			for(i = 0; i<3;i++){
								
				if(current_sensor[i]>=150){
					if(i == 0 || i == 1){ //pressuresensors
						value = ( ((double) current_sensor[i]) * 11.0) / 2024.0; //constants for calculating current
						printf("%1.2f,", value);
					}
					else{	//stackvoltage
					
						printf("err,");
						//printf("%d,",current_sensor[i]);
						//value = ( ((double) current_sensor[i]) * 11.0) / 2024.0; //constants for calculating current
					}
					

				}
				else{ //less than 4mA in current loop, broken sensor or loop
				
					printf("err,");
				}
			}

			USART_Transmit_debug('O');
			USART_Transmit_debug(',');
			for(i = 0; i<5;i++){
				printf("%x,", output[i]);
			}

			USART_Transmit_debug('E');
			USART_Transmit_debug(',');
			for(i = 0; i<1;i++){
				printf("%d,", cell_undervoltage);
			}
			
			USART_Transmit_debug('\n');
			USART_Transmit_debug('\r');		



			//send can messages

			SendDataOverCAN(cell_voltage, temperature, current_sensor, output);


			measure_counter=0;
		}


		//purge
		if(purge){
			SET_BIT(PORTD, PD3);
			_DELAY_MS(250);
			CLEAR_BIT(PORTD, PD3);
			purge = 0;
		
		}

		//turn on
		if(turn_on){
			
			SET_BIT(PORTE, PE3);
			_DELAY_MS(500);
			//purge
			SET_BIT(PORTD, PD3);

			_DELAY_MS(200);

			CLEAR_BIT(PORTD, PD3);
			turn_on = 0;
		
		}

		//turn off
		if(turn_off){
		
			CLEAR_BIT(PORTE, PE3);
			turn_off = 0;
		}



		//uart-debug commands

		data = USART_Receive_debug();

		switch (data){
		case '1':
			CLEAR_BIT(PORTE, PE3);
		break;
		case '2':
			SET_BIT(PORTE, PE3);
		break;
		case '3':
			CLEAR_BIT(PORTD, PD3);
		break;
		case '4':
			SET_BIT(PORTD, PD3);
		break;
		case '5':
			CLEAR_BIT(PORTD, PD2);
		break;
		case '6':
			
		break;
		case '7':
			CLEAR_BIT(PORTE, PE4);
		break;
		case '8':
			SET_BIT(PORTE, PE4);
		break;
		case '9':
			CLEAR_BIT(PORTE, PE5);
		break;	
		case '0':
			SET_BIT(PORTE, PE5);
		break;	

		case'p': //purge
			SET_BIT(PORTD, PD3);
			_DELAY_MS(250);
			CLEAR_BIT(PORTD, PD3);
	
		break;
		case'f': 
			interval = 1;
		break;	
		case'm': 
			interval = 5;
		break;	
		case's': 
			interval = 10;
		break;
		case't': 
		
		break;
		case'r':
			init_battery_monitor();
			cell_undervoltage = 0;
		break;
		}


		//check for low cell_voltage and turn off load
		//disabled for the moment
		/*
		for(i = 0; i<46;i++){
			if(cell_voltage[i] < 330){
				cell_undervoltage = i+1;
				CLEAR_BIT(PORTD, PD2);
				break;
			}
		}
		*/
/*
		//turn on fan if voltage_sum <=33 and no cell_undervoltage condition

		if(voltage_sum <= 33.0 && cell_undervoltage == 0){
			SET_BIT(PORTE, PE5);
		}
		else{
			CLEAR_BIT(PORTE, PE5);
		}

		//init batterymonitor again if voltage_sum is too high (means that the unit has reset itself because of low supply or watchdog
		//also turn off fan
		if(voltage_sum >= 100.0){
			init_battery_monitor();
			CLEAR_BIT(PORTE, PE5);
			
		}
*/
		//send and receive CAN-messages


		while(new_can_frame()){//receive all new can-frames
			recv = read_can_frame();
			next_can_frame();
			if(recv->id == 0x3C0){
				if(recv->data[0]){
					purge = 1;		
			
				}
			}
			else if(recv->id == 0x3C1){
				
				if(recv->data[0]){
					turn_on = 1;
				}
				else{
					turn_on = 0;
				}
				
				
			}	
			else if(recv->id == 0x3C9){
				
				
				
				
			}	
					
				
			 

		}


	
		//send msg over CAN
		SendStatusMsg(0x0C);
	

		_DELAY_MS(10);
	}
}


void SendStatusMsg(unsigned char status){
		
	static unsigned int last_system_ticks = 0;
	if(last_system_ticks != get_system_ticks()){
		union can_frame send;
		send.id = 0x7C0;			
		send.length = 1;
		send.rbr = 0;
		send.rb0 = 0;
		send.data[0] = status;
		send_can_frame(&send);

		//status to dashboard
		send.id = 0x750;
		send.length = 8;
		send.rbr = 0;
		send.rb0 = 0;
		send.data[0] = status;
		int i;
		for(i=1;i<8;i++){
			send.data[i] = 0;
		}
		send_can_frame(&send);		

		last_system_ticks = get_system_ticks();
	}
	
}


void SetFanSpeed(unsigned char speed){

		
	if(speed == 0){
		CLEAR_BIT(PORTD, PD2);
		Timer0RemoveCallback(FanPWM);
	}else{
		fuelcell.fan_dutycycle = speed;	
		Timer0RegisterCallback(FanPWM, 1);
	}

}

void FanPWM(void){
	static unsigned char counter = 0;
	
	unsigned char fan_on = 2;

	
	counter++;
	
	//turn on
	if(counter>=122){
		counter = 0;
		fan_on = 1;
	}
	
	//turn off
	if(counter == fuelcell.fan_dutycycle){
		fan_on = 0;
		
	}
	
	if(fan_on == 1){
		
	}	
	else if(fan_on == 0){
		
	}

}









