/* rev 1.0 12/2009 				written by Anders Guldahl			     */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <stdio.h>

#include "voltage_adc.h"
#include "macro.h"

#define avg_number 32

void init_adc(){

	DDRF &= ~((1<<PF0)|(1<<PF1)|(1<<PF2));
	PORTF &= ~((1<<PF0)|(1<<PF1)|(1<<PF2));

	//disable digital inputbuffer
	DIDR0 |= (1<<ADC2D)|(1<<ADC1D)|(1<<ADC0D);

	// AVcc as reference.
	ADMUX |= (1 << REFS0);
	// ADC Prescaler 64.
	ADCSRA |= (1 << ADPS2)|(1 << ADPS1);


	// Enable ADC.
	ADCSRA |= (1 << ADEN);


	return;
}


int read_mcu_voltage(void){

	//read 0V to reset the sample and hold capacitor in avr-adc,
	//will be error readings if this is not done
	read_gnd_voltage();

	//select internal 1.1V bandgap as input
	ADMUX &= ~0x01;
	ADMUX |= 0x1e;


	int i;
	int readvalue = 0;
	int average = 0;

	for(i = 0; i<avg_number;i++){
		//start conversion
		ADCSRA |= (1 << ADSC);
		while(ADCSRA & (1 << ADSC)); //wait for adc conversion complete

		readvalue = ADCL;
		readvalue += ((ADCH & 0x00FF)<<8);

		average += readvalue;

	}

	average = average/avg_number;

	return average;
}

int read_board_voltage(void){

	//read 0V to reset the sample and hold capacitor in AVR-ADC,
	//will be error readings if this is not done
	read_gnd_voltage();

	//adc1 as input
	ADMUX &= ~(0x1e);
	ADMUX |= 0x01;

	int i;
	int readvalue = 0;
	int average = 0;

	for(i = 0; i<avg_number;i++){
		//start conversion
		ADCSRA |= (1 << ADSC);
		while(ADCSRA & (1 << ADSC)); //wait for adc conversion complete

		readvalue = ADCL;
		readvalue += ((ADCH & 0x00FF)<<8);

		average += readvalue;

	}

	average = average/avg_number;

	return average;

}

int read_input_voltage(void){

	//read 0V to reset the sample and hold capacitor in avr-adc,
	//will be error readings if this is not done
	read_gnd_voltage();

	//adc2 as input
	ADMUX &= ~0x1d;
	ADMUX |= 0x02;

	int i;
	int readvalue = 0;
	int average = 0;

	for(i = 0; i<avg_number;i++){
		//start conversion
		ADCSRA |= (1 << ADSC);
		while(ADCSRA & (1 << ADSC)); //wait for adc conversion complete

		readvalue = ADCL;
		readvalue += ((ADCH & 0x00FF)<<8);

		average += readvalue;

	}

	average = average/avg_number;

	return average;
}

int read_channel(unsigned char channel){
	
	
	//read 0V to reset the sample and hold capacitor in avr-adc,
	//will be error readings if this is not done
	read_gnd_voltage();

	//channel as input
	switch(channel){
		case 0:
			ADMUX &= ~(0x1f);
		break;
		case 1:
			ADMUX &= ~(0x1e);
			ADMUX |= 0x01;
		break;
		case 2:
			ADMUX &= ~0x1d;
			ADMUX |= 0x02;
		break;
		case 3:
			ADMUX &= ~0x1c;
			ADMUX |= 0x03;
		break;
	
	}


	int i;
	int readvalue = 0;
	int average = 0;

	for(i = 0; i<avg_number;i++){
		//start conversion
		ADCSRA |= (1 << ADSC);
		while(ADCSRA & (1 << ADSC)); //wait for adc conversion complete

		readvalue = ADCL;
		readvalue += ((ADCH & 0x00FF)<<8);

		average += readvalue;

	}

	average = average/avg_number;

	return average;


}


void read_gnd_voltage(void){

	//select internal GND as input
	ADMUX &= ~0x00;
	ADMUX |= 0x1f;

	//start conversion
	ADCSRA |= (1 << ADSC);
	while(ADCSRA & (1 << ADSC)); //wait for adc conversion complete


	return;
}
