/*-----------------------------------------------------------------------*/
/* Low level disk I/O module skeleton for FatFs     (C)ChaN, 2007        */
/*-----------------------------------------------------------------------*/
/* rev 1.1 12/2009 				edited by Anders Guldahl			     */
/*-----------------------------------------------------------------------*/

#include "diskio.h"
#include "SD_routines.h"
#include "gps.h"
#include "../common_files/status_structs.h"

/*-----------------------------------------------------------------------*/

#define SD		0

/*-----------------------------------------------------------------------*/
/* Inidialize a Drive                                                    */

DSTATUS disk_initialize (BYTE drv){
	//we only have SD-card as drive
	int result;
	result = SD_init();
	if(result == 0){
		return 0;
	}
	return STA_NOINIT;
}



/*-----------------------------------------------------------------------*/
/* Return Disk Status                                                    */

DSTATUS disk_status (
	BYTE drv
)
{
	int result;
	result = SD_disk_status();
	if(result == 0){
		return 0;
	}
	return STA_NOINIT;
}



/*-----------------------------------------------------------------------*/
/* Read Sector(s)                                                        */

DRESULT disk_read (
	BYTE drv,		/* Physical drive number (0..) */
	BYTE *buff,		/* Data buffer to store read data */
	DWORD sector,	/* Sector address (LBA) */
	BYTE count		/* Number of sectors to read (1..255) */
)
{

	int result;
	result = SD_disk_read(buff, sector, count);
	if(result == 0){
		return RES_OK;
	}
	return RES_PARERR;
}



/*-----------------------------------------------------------------------*/
/* Write Sector(s)                                                       */

#if _READONLY == 0
DRESULT disk_write (
	BYTE drv,			/* Physical drive nmuber (0..) */
	const BYTE *buff,	/* Data to be written */
	DWORD sector,		/* Sector address (LBA) */
	BYTE count			/* Number of sectors to write (1..255) */
)
{

	int result;


	result = SD_disk_write(buff, sector, count);
	if(result == 0){
		return RES_OK;

	}


	return RES_PARERR;
}
#endif /* _READONLY */



/*-----------------------------------------------------------------------*/
/* Miscellaneous Functions                                               */

DRESULT disk_ioctl (
	BYTE drv,		/* Physical drive nmuber (0..) */
	BYTE ctrl,		/* Control code */
	void *buff		/* Buffer to send/receive control data */
)
{
	DRESULT res;

	unsigned long * retur;
	retur = (unsigned long*) buff;



	res = RES_PARERR;

	switch(ctrl){
	case CTRL_SYNC:
		res = RES_OK;
	break;
	case GET_SECTOR_COUNT:
		*retur = 1950208;

		res = RES_OK;
	break;
	case GET_SECTOR_SIZE:
		*retur = 512;

		res = RES_OK;
	break;
	case GET_BLOCK_SIZE:
		*retur = 128;

		res = RES_OK;
	break;


	}

	return res;
}

DWORD get_fattime (void){

	return  ((DWORD)((WORD)(rtc_struct.Year) + 20) << 25)
		   | ((DWORD)rtc_struct.Month << 21)
		   | ((DWORD)rtc_struct.Day << 16)
		   | ((DWORD)rtc_struct.Hours << 11)
		   | ((DWORD)rtc_struct.Minutes << 5)
		   | ((DWORD)rtc_struct.Seconds >> 1);

}

