/* rev 1.0 12/2009 				written by Anders Guldahl			     */

#include <stdio.h>
#include <string.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

#include "macro.h"
#include "uart_driver.h"
#include "uart_debug.h"
#include "status_led.h"
#include "voltage_adc.h"
#include "can.h"
#include "telit.h"
#include "SD_routines.h"
#include "SPI_routines.h"
#include "ff.h"
#include "gps.h"
#include "diskio.h"
#include "main.h"
#include "../common_files/eeprom.h"
#include "../common_files/status_structs.h"

#define SOCKET_OPEN 0
#define SOCKET_SUSPENDED 1
#define SOCKET_CLOSED 3
#define SLEEP 4



// extern structs

BOARD_STATUS board_status;
RTC_CURRENT rtc_struct;
GPS_CURRENT gps_struct;
FUELCELL fuelcell;


//initialize module
unsigned char init(void){

	board_status.reset_source = MCUSR;
	MCUSR = 0;
	wdt_disable();
	
	eepromResetBootByte();
	add_to_PowerOnCount();

	USART_Init_debug();
	USART_Init();
	init_can();
	init_adc();
	spi_init();
	init_status_led();

	init_telit_pwr_switch();

	heart_beat(1);
	set_status_led(BLUE);

	

	sei();
	return 0;
}





int main(){


	init();
/*
	//do nothing if only usb power is connected (input voltage-adc-read less than 400)
	if(read_input_voltage() < 400){
		printf("usb connected\n\r");
		set_status_led(MAGENTA);

		PORTG |= (1<<PG0);
		_DELAY_MS(1300);
		PORTG &= ~(1<<PG0);

		while(read_input_voltage() < 400){
			_DELAY_MS(2000);

		}


	}
*/
/*
	//try to power on while telit not answering
	while(telit_attention() == 0){
		telit_pwr_switch_on();
	}
*/
	_DELAY_MS(1000);

	if(telit_attention() == 0){
		PORTG |= (1<<PG0);
		_DELAY_MS(1300);
		PORTG &= ~(1<<PG0);
		
	}

	set_status_led(RED);
	
	

	unsigned char data;


	union can_frame send;
	union can_frame msg;
	union can_frame *recv;
	recv = &msg;



	int try_connect_counter = 10;
	int send_counter = 0;
	int telit_restart = 0;
	int gprs_connection_loss = 0;
	int byteswritten;
	char output_string[400];
	char input_string[200];
	char filename[12];
	unsigned char state = SOCKET_CLOSED;
	char result = 0;




	// init filesystem, make new filename and open/make file
	FATFS FileSystemObject;
	FIL logFile;
	int res;

	sprintf(filename,"log%d.txt",board_status.times_powered_on);

	res = f_mount(0, &FileSystemObject);
	res = f_open(&logFile, filename, FA_READ | FA_WRITE | FA_OPEN_ALWAYS);





	printf("Times powered on: %d reset_source %x\n\r", board_status.times_powered_on, board_status.reset_source);

	unsigned int last_system_ticks = 0;
	
	int i;

	while(1){

		
		//printf("%ld\n\r",get_fattime());

		board_status.mcu_voltage = read_mcu_voltage();
		board_status.board_voltage = read_board_voltage();
		board_status.input_voltage = read_input_voltage();

		
		printStatusString(output_string);



		//sprintf(output_string,"%d mcu %d,board %d,input %d\r\n",send_counter,board_status.mcu_voltage,board_status.board_voltage,board_status.input_voltage);
		
		//sprintf_P(output_string, )

		//printf(output_string);
		
		
		
		if(last_system_ticks != get_system_ticks()){
			GPS_update();
			printStatusString(output_string);


			if(res==FR_OK){//if last SD-card write was successfull try again
				cli();
				res = f_write(&logFile,output_string,strlen(output_string),&byteswritten);
				f_sync(&logFile);
				sei();
				printf("sd card success\n\r");
			}else{
				printf("sd card error\n\r");
			}


/*
			if(gprs_is_connected()){ //we have gprs available
				set_status_led(GREEN);


				switch(state){
					case SOCKET_SUSPENDED:
						heart_beat(1);
						result = 0;
						result = resume_socket();

						uputs(output_string);



						send_counter++;
						result &= suspend_socket();
						//printf("result %d state %d\n\r",result,state);
						if(!result){

							state = SOCKET_CLOSED;
							close_socket();
						}

					break;
					case SOCKET_CLOSED:
						fast_breathing(1);
						result = 0;

						result = open_socket();

						uputs("tcp_established\n\r");

						result &= suspend_socket();
						//printf("result %d state %d\n\r",result,state);
						if(result){
							state = SOCKET_SUSPENDED;
						}

					break;
					case SLEEP:

						slow_breathing(1);

					break;
				}


			}
			else if(telit_attention()){//we don't have gprs available, is the telit-module available at all?
				set_status_led(RED);
				heart_beat(1);
				if(try_connect_counter >= 10){

					gprs_connect();

					try_connect_counter = 0;
				}
				else{
					try_connect_counter++;
				}


			}
			else{//telit-module not answering
				heart_beat(1);
				telit_pwr_switch_on();
				set_status_led(BLUE);

				telit_restart++;
			}

*/
			



			last_system_ticks = get_system_ticks();
		}



		//quick hack uart-debug commands, mostly changing LED-color, useful for testpurposes
/*
		data = USART_Receive_debug(NULL);

		switch (data){
		case'1':
			heart_beat(1);
		break;
		case'2':
			slow_breathing(1);
		break;
		case'3':
			fast_breathing(1);
		break;
		case'4':
			set_status_led(RED);
		break;
		case'5':
			set_status_led(GREEN);
		break;
		case'6':
			set_status_led(BLUE);
		break;
		case'7':
			set_status_led(YELLOW);
		break;
		case'8':
			while(1);
		break;
		case'9':

		break;
		case'q':

		break;
		case'w':
			set_status_led(LED_OFF);
		break;
		case't':


		break;
		case'd':


		break;
		case'r':
			state = SOCKET_CLOSED;
			close_socket();
		break;
		case'a':
			state = SLEEP;
			close_socket();
		break;
		}

*/





		//send and receive CAN-messages

		while(new_can_frame()){//receive all new can-frames
			recv = read_can_frame();
			next_can_frame();

			switch(recv->id){
				case 0x012:
					state = SOCKET_CLOSED;
					close_socket();
				break;
				case 0x011:
					state = SLEEP;
					close_socket();
				break;
				
				case 0x700:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i] |= recv->data[i*2+1];
					}
				break;

				case 0x701:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+4] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+4] |= recv->data[i*2+1];
					}
				break;

				case 0x702:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+8] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+8] |= recv->data[i*2+1];
					}
				break;

				case 0x703:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+12] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+12] |= recv->data[i*2+1];
					}
				break;		

				case 0x704:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+16] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+16] |= recv->data[i*2+1];
					}
				break;

				case 0x705:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+20] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+20] |= recv->data[i*2+1];
					}
				break;

				case 0x706:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+24] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+24] |= recv->data[i*2+1];
					}
				break;

				case 0x707:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+28] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+28] |= recv->data[i*2+1];
					}
				break;

				case 0x708:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+32] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+32] |= recv->data[i*2+1];
					}
				break;

				case 0x709:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+36] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+36] |= recv->data[i*2+1];
					}
				break;

				case 0x70A:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+40] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+40] |= recv->data[i*2+1];
					}
				break;

				case 0x70B:
					for(i = 0;i<4;i++){
						fuelcell.cell_voltage[i+44] = recv->data[i*2]<<8;
						fuelcell.cell_voltage[i+44] |= recv->data[i*2+1];
					}
				break;

				case 0x70C:
					for(i = 0;i<4;i++){
						fuelcell.temperature[i] = recv->data[i*2]<<8;
						fuelcell.temperature[i] |= recv->data[i*2+1];
					}
				break;

				case 0x70D:
					for(i = 0;i<4;i++){
						fuelcell.temperature[i+4] = recv->data[i*2]<<8;
						fuelcell.temperature[i+4] |= recv->data[i*2+1];
					}
				break;

				case 0x70E:
					for(i = 0;i<4;i++){
						fuelcell.temperature[i+8] = recv->data[i*2]<<8;
						fuelcell.temperature[i+8] |= recv->data[i*2+1];
					}
	
				break;

				case 0x70F:

					fuelcell.pressure_sensor[0] = recv->data[0]<<8;
					fuelcell.pressure_sensor[0] |= recv->data[1];
					fuelcell.pressure_sensor[1] = recv->data[2]<<8;
					fuelcell.pressure_sensor[1] |= recv->data[3];
					fuelcell.pressure_sensor[2] = recv->data[4]<<8;
					fuelcell.pressure_sensor[2] |= recv->data[5];


				break;

				case 0x710:
				
					for(i = 0;i<5;i++){
						fuelcell.output[i] = recv->data[i];
					}
					
				break;

				case 0x782:
						
					//format the SD-card
					res = f_mkfs(0,0,0);
					if(res !=FR_OK) {
				    	printf("format error %d\n\r",res);
				  	}else{
						printf("format successfull\n\r");
					}

				break;


			}

		}




		SendStatusMsg(0x0C);

		_DELAY_MS(10);
	}
}//end of main



char * printStatusString(char output_string[]){

	int j;
	
	char temp_string[20];
	double voltage_sum = 0;
	double value;
	int integer_value;

	sprintf(output_string,"V,");

	for(j = 0;j<46;j++){
		

		value = ((double) fuelcell.cell_voltage[j])*0.0015;
		sprintf(temp_string,"%1.2f,",value);
		voltage_sum+=value;
		output_string = strcat(output_string, temp_string);

	
	}


	output_string = strcat(output_string, "S,");
	sprintf(temp_string, "%2.2f,", ((double)fuelcell.pressure_sensor[2]*37)/512);
	output_string = strcat(output_string, temp_string);
	
	output_string = strcat(output_string, "T,");

	for(j = 0;j<12;j++){
		
		integer_value = fuelcell.temperature[j];
		sprintf(temp_string,"%d,",integer_value);
		output_string = strcat(output_string, temp_string);

	
	}	

	output_string = strcat(output_string, "P,");

	for(j = 0;j<3;j++){
		if(fuelcell.pressure_sensor[j]>=150){
			if(j == 0 || j == 1){ //pressuresensors
				value = ( ((double) fuelcell.pressure_sensor[j]) * 11.0) / 2024.0; //constants for calculating current
				sprintf(temp_string,"%1.2f,",value);
				output_string = strcat(output_string, temp_string);

			}
			else{	//other loop sensor
		

			}
		

		}
		else{ //less than 4mA in current loop, broken sensor or loop
	
			output_string = strcat(output_string, "err,");
		}
	}


	output_string = strcat(output_string, "O,");
	
	for(j = 0;j<5;j++){
		sprintf(temp_string,"%d,",fuelcell.output[j]);
		output_string = strcat(output_string, temp_string);
	
	}
	
	output_string = strcat(output_string, "\n");
	return output_string;

}


void SendStatusMsg(unsigned char status){
		
	static unsigned int last_system_ticks = 0;
	if(last_system_ticks != get_system_ticks()){
		union can_frame send;
		send.id = 0x7E0;			
		send.length = 1;
		send.rbr = 0;
		send.rb0 = 0;
		send.data[0] = status;
		send_can_frame(&send);

		//status to dashboard
		send.id = 0x762;
		send.length = 8;
		send.rbr = 0;
		send.rb0 = 0;
		send.data[0] = status;
		int i;
		for(i=1;i<8;i++){
			send.data[i] = 0;
		}
		send_can_frame(&send);	
			
		//send current time over CAN
		send.id = 0x763;
		send.length = 8;
		send.rbr = 0;
		send.rb0 = 0;
		send.data[0] = 0xff;
		send.data[1] = 0xff;

		send.data[2] = rtc_struct.Hours;
		send.data[3] = rtc_struct.Minutes;
		send.data[4] = rtc_struct.Seconds;

		send.data[5] = rtc_struct.Day;
		send.data[6] = rtc_struct.Month;
		send.data[7] = rtc_struct.Year;

		send_can_frame(&send);

		send.id = 0x764;
		send.length = 8;
		send.rbr = 0;
		send.rb0 = 0;
		send.data[0] = gps_struct.Latitude[0];
		send.data[1] = gps_struct.Latitude[1];
		send.data[2] = gps_struct.Latitude[2];
		send.data[3] = gps_struct.Latitude[3];

		send.data[4] = gps_struct.Longitude[0];
		send.data[5] = gps_struct.Longitude[1];
		send.data[6] = gps_struct.Longitude[2];
		send.data[7] = gps_struct.Longitude[3];

		send_can_frame(&send);


		last_system_ticks = get_system_ticks();
	}
	
}
