#include <stdio.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

#include "macro.h"
#include "uart_driver.h"
#include "uart_debug.h"
#include "status_led.h"
#include "voltage_adc.h"
#include "can.h"
#include "telit.h"
#include "SD_routines.h"
#include "SPI_routines.h"


#define SOCKET_OPEN 0
#define SOCKET_SUSPENDED 1
#define SOCKET_CLOSED 3
#define SLEEP 4

typedef struct{

	int mcu_voltage;
	int board_voltage;
	int input_voltage;

}board_status;



unsigned char init(void){



	USART_Init_debug();
	USART_Init();
	init_can();
	init_adc();
	spi_init();
	init_status_led();

	SD_init();


	init_telit_pwr_switch();

	heart_beat(1);
	set_status_led(BLUE);


	sei();
	return 0;
}





int main(){


	init();


	if(read_input_voltage() < 400){
		printf("usb connected\n\r");
		set_status_led(MAGENTA);

		PORTG |= (1<<PG0);
		_DELAY_MS(1300);
		PORTG &= ~(1<<PG0);

		while(read_input_voltage() < 400){
			_DELAY_MS(2000);

		}


	}



	while(telit_attention() == 0){
		telit_pwr_switch_on();
	}

	set_status_led(RED);

	board_status status;




	unsigned char data;


	union can_frame send;
	union can_frame msg;
	union can_frame *recv;
	recv = &msg;





	int try_connect_counter = 10;
	int send_counter = 0;
	char output_string[200];
	char input_string[200];

	unsigned char state = SLEEP;
	char result = 0;


	int SD_result;



	int k;
	for(k = 0;k<512;k++){

		buffer[k] = 'r';
	}


	SD_result = SD_writeSingleBlock(0);

	if(SD_result == 0){
		set_status_led(YELLOW);

		_DELAY_MS(2000);
	}
	else{
		set_status_led(CYAN);
		_DELAY_MS(2000);
	}

	for(k = 0;k<512;k++){

		buffer[k] = 'a';
	}

	SD_result = SD_readSingleBlock(0);
	int buf_status = 0;
	for(k = 0;k<512;k++){

		if(buffer[k] != 'r'){
			buf_status = 1;

		}

	}

	if(buf_status == 0){
		set_status_led(YELLOW);
		HIGH_SPEED_SPI;
		_DELAY_MS(2000);
	}
	else{
		set_status_led(CYAN);
		_DELAY_MS(2000);
	}


	while(1){



		status.mcu_voltage = read_mcu_voltage();
		status.board_voltage = read_board_voltage();
		status.input_voltage = read_input_voltage();

		sprintf(output_string,"%d mcu %d,board %d,input %d, can-id %d, can-data %d\n\r",send_counter,status.mcu_voltage,status.board_voltage,status.input_voltage, recv->id, recv->data[0]);
		printf(output_string);


		if(gprs_is_connected()){
			set_status_led(GREEN);


			switch(state){
				case SOCKET_SUSPENDED:
					heart_beat(1);
					result = 0;
					result = resume_socket();

					uputs(output_string);



					send_counter++;
					result &= suspend_socket();
					printf("result %d state %d\n\r",result,state);
					if(!result){

						state = SOCKET_CLOSED;
						close_socket();
					}

				break;
				case SOCKET_CLOSED:
					fast_breathing(1);
					result = 0;

					result = open_socket();

					uputs("tcp_established\n\r");

					result &= suspend_socket();
					printf("result %d state %d\n\r",result,state);
					if(result){
						state = SOCKET_SUSPENDED;
					}

				break;
				case SLEEP:

					slow_breathing(1);

				break;
			}


		}
		else if(telit_attention()){
			set_status_led(RED);
			heart_beat(1);
			if(try_connect_counter >= 10){

				gprs_connect();

				try_connect_counter = 0;
			}
			else{
				try_connect_counter++;
			}


		}
		else{
			heart_beat(1);
			set_status_led(BLUE);
		}







		data = USART_Receive_debug();

		switch (data){
		case'1':
			heart_beat(1);
		break;
		case'2':
			slow_breathing(1);
		break;
		case'3':
			fast_breathing(1);
		break;
		case'4':
			set_status_led(RED);
		break;
		case'5':
			set_status_led(GREEN);
		break;
		case'6':
			set_status_led(BLUE);
		break;
		case'7':
			set_status_led(YELLOW);
		break;
		case'8':
			while(1);
		break;
		case'9':

		break;
		case'q':

		break;
		case'w':
			set_status_led(LED_OFF);
		break;
		case't':


		break;
		case'd':


		break;
		case'r':
			state = SOCKET_CLOSED;
			close_socket();
		break;
		case'a':
			state = SLEEP;
			close_socket();
		break;
		}










		if(new_can_frame()){
			recv = read_can_frame();
			next_can_frame();
			printf("can id %x, data %x\n\r", recv->id,recv->data[0]);
		}



		send_can_message(0x190, 0x01, &send);





		_DELAY_MS(500);
	}
}


