
/************************************************************************* 
* description:    low level C functions to talk to the canbus-chip over spi
* 
**************************************************************************/
#include <avr/io.h>
#include <avr/interrupt.h>

#include "spi.h"


void SPI_init(void){
	/* Set MOSI and SCK and pb0 output, all others input */
	DDRB |= (1<<DDB3)|(1<<DDB2)|(1<<DDB1)|(1<<DDB0);
	/* Enable SPI, Master, set clock rate fck/2 */
	SPCR = (1<<SPE)|(1<<MSTR);
	SPSR |= (1<<SPI2X);
	/* Set chipselect high*/
	PORTB |= (1<<SD_CS);
}

void SPI_tx(unsigned char cData){
	
	PORTB &= ~(1<<SD_CS);

	SPI_tx_noCS(cData);

	PORTB |= (1<<SD_CS);
}

unsigned char SPI_rx(void){

	PORTB &= ~(1<<SD_CS);
	unsigned char result;
	result = SPI_rx_noCS();

	PORTB |= (1<<SD_CS);
	return result;
}

//transmitroutine without chipselect, used to transmit several bytes
void SPI_tx_noCS(unsigned char cData){
	
	SPDR = cData;
	while(!(SPSR & (1<<SPIF)));
}


//receiveroutine without chipselect, used to receive several bytes
unsigned char SPI_rx_noCS(void){

	SPI_tx_noCS(0xff);
	
	return SPDR;
}

