/* rev 1.0 12/2009 				written by Anders Guldahl			     */
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include "macro.h"
#include "status_led.h"

#define HEART 0
#define SLOW_B 1
#define FAST_B 2
#define NOFX 3

volatile unsigned char mode = 0;
volatile unsigned char current_color = 0;
volatile unsigned int system_ticks = 0;

void init_status_led(void){

	//set as output
	DDRB |= (1<<PB4)|(1<<PB5)|(1<<PB6);
	PORTB |= (1<<PB4)|(1<<PB5)|(1<<PB6);

	set_status_led(LED_OFF);

	//init 16bit timer
	//Mode 14, Fast PWM (se s137 i datablad)
	SET_BIT(TCCR1B, WGM13);
	SET_BIT(TCCR1B, WGM12);
	SET_BIT(TCCR1A, WGM11);
	CLEAR_BIT(TCCR1A, WGM10);

	//Setter pwm-utgang lav p compare match,
	//for output compare a, b og c
	SET_BIT(TCCR1A, COM1A1);
	CLEAR_BIT(TCCR1A, COM1A0);
	SET_BIT(TCCR1A, COM1B1);
	CLEAR_BIT(TCCR1A, COM1B0);

	//Sett CSn2:0 for prescaler = clk / 1024
	SET_BIT(TCCR1B, CS12);
	CLEAR_BIT(TCCR1B, CS11);
	CLEAR_BIT(TCCR1B, CS10);

	//Sett toppverdi (periode)
	ICR1H = 0;
	ICR1L =  0xff;

	//init 8bit timer

	TCCR2A = (1 << WGM21) | (1 << WGM20) | (1 << COM2A1) | (1 << CS22) | (1 << CS21);

	TIMSK2 = (1 << TOIE2);

	set_status_led(LED_OFF);
}


void set_status_led(char color){

	current_color = color;


}

void set_status_rgb(unsigned char red, unsigned char green, unsigned char blue){


	OCR1AL = 0xff - green;
	OCR1BL = 0xff - red;
	OCR2A = 0xff - blue;

	return;


}

void change_current_color(unsigned char value){


	//inverted logic
	switch(current_color){
	case RED:
		OCR1AL = 0xff;
		OCR1BL = 0xff - value;
		OCR2A = 0xff;

	break;
	case GREEN:
		OCR1AL = 0xff - value;
		OCR1BL = 0xff;
		OCR2A = 0xff;
	break;
	case BLUE:
		OCR1AL = 0xff;
		OCR1BL = 0xff;
		OCR2A = 0xff - value;
	break;
	case YELLOW:
		OCR1AL = 0xff - (value>>1);
		OCR1BL = 0xff - value;
		OCR2A = 0xff;
	break;
	case MAGENTA:
		OCR1AL = 0xff;
		OCR1BL = 0xff - value;
		OCR2A = 0xff - value;
	break;
	case CYAN:
		OCR1AL = 0xff-value;
		OCR1BL = 0xff;
		OCR2A = 0xff - (value>>1);
	break;
	case WHITE:

		OCR1AL = 0xff - value;
		OCR1BL = 0xDF - (value*0.88);
		OCR2A = OCR1BL;
	break;
	case ORANGE:
		OCR1AL = 0xff - value;
		OCR1BL = 0xff - value;
		OCR2A = 0xff;
	break;
	case LED_OFF:
		OCR1AL = 0xff;
		OCR1BL = 0xff;
		OCR2A = 0xff;
	break;


	}


}

void heart_beat(char on){

	if(on){
		mode = HEART;
	}
	else{
		mode = NOFX;

	}
}

void slow_breathing(char on){

	if(on){
		mode = SLOW_B;
	}
	else{
		mode = NOFX;

	}

}

void fast_breathing(char on){

	if(on){
		mode = FAST_B;
	}
	else{
		mode = NOFX;

	}

}

unsigned int get_system_ticks(void){

	DISABLE_TM2OVF_INT;
	unsigned int ticks = system_ticks;
	ENABLE_TM2OVF_INT;
	return ticks;
}


volatile unsigned char i = 0;
volatile int counter = 0;
volatile char count_dir = 1;
unsigned char value = 0;

//lookup-tables for smooth heartbeat-blinking and breathing-effects like the standby-light on mac-books
const unsigned char heart_lookup[] = {
1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,3,11,20,30,50,80,113,141,173,211,255,211,173,141,113,
80,50,30,20,11,20,30,50,80,113,141,173,211,
255,211,173,141,113,80,50,30,20,11,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
};



const unsigned char breathing_lookup[] = {
2,2,2,2,2,2,2,2,2,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,3,3,3,4,4,5,6,6,7,8,10,
11,12,14,16,18,20,22,25,27,30,34,37,41,45,49,54,59,64,70,76,83,90,
97,105,113,122,131,141,151,162,173,185,198,211,225,240,255
};



unsigned char interrupt_counter = 0;


ISR(TIMER2_OVF_vect){



	i++;

	interrupt_counter++;
	if(interrupt_counter>=61){//every half second
		system_ticks++;
		interrupt_counter=0;
		

	}
	

	switch(mode){
		case HEART:
			if(i > 1 ){
				i = 0;

				if(count_dir == 1){
					counter++;
				}
				else{
					counter--;
				}
				if(counter>73){
					count_dir = 0;
				}
				if(counter<=0){
					count_dir = 1;

				}
				change_current_color(heart_lookup[counter]);

			}




		break;
		case SLOW_B:
			if(i > 4 ){
				i = 0;

				if(count_dir == 1){
					counter+=2;
				}
				else{
					counter--;
				}
				if(counter>72){
					count_dir = 0;
				}
				if(counter<=0){
					count_dir = 1;

				}

				change_current_color(breathing_lookup[counter]);

			}

		break;
		case FAST_B:
			if(i > 1 ){
				i = 0;

				if(count_dir == 1){
					counter+=2;
				}
				else{
					counter--;
				}
				if(counter>72){
					count_dir = 0;
				}
				if(counter<=0){
					count_dir = 1;

				}

				change_current_color(breathing_lookup[counter]);
			}
		break;

		case NOFX:
			change_current_color(0xff);
		break;
	}




}








