/* rev 1.0 12/2009 				written by Anders Guldahl			     */
#include <stdio.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>

#include "macro.h"
#include "uart_driver.h"
#include "uart_debug.h"
#include "telit.h"
#include "integer.h"

// strings


//keep all text in flash, saves RAM on runtime
const char cr_P[] PROGMEM = "\n";
const char line_P[] PROGMEM = "------\n";

const char menu_option_P[] PROGMEM = " - ";
const char prompt_P[] PROGMEM = "key>";
const char error_unknown_command_P[] PROGMEM = "unknown key: %c\n";
const char got_P[] PROGMEM = "got: %s\n";

const char ok_P[] PROGMEM = " --> OK\n";
const char no_response_P[] PROGMEM = " --> NO RESPONSE\n";
const char no_fix_P[] PROGMEM = "no fix\n";
const char error_P[] PROGMEM = "ERROR\n";
const char interactive_P[] PROGMEM = "INTERACTIVE\n";
const char run_P[] PROGMEM = "RUN\n";
const char wait_P[] PROGMEM = "waiting ...\n";

// modem strings
const char AT_P[] PROGMEM       = "AT";					// say hello
const char OK_P[] PROGMEM       = "\r\nOK\r\n";			// response

const char ATCMEE_P[] PROGMEM   = "AT+CMEE=2";			// extended error format


const char ATGPSR_P[] PROGMEM   = "AT$GPSR=2";			// reset GPS, warm start
const char ATGPSACP_P[] PROGMEM = "AT$GPSACP";			// position request

const char ATCGDCONT_P[] PROGMEM = "AT+CGDCONT=1,\"IP\",\"internet\",\"0.0.0.0\",0,0";

const char ATSCFG_P[] PROGMEM = "AT#SCFG=1,1,400,90,600,50";
const char ATCGATT_P[] PROGMEM = "AT+CGATT?";  //gprs attached ?

const char ATSD_P[] PROGMEM = "AT#SD=1,0,45987,\"ecometry.dyndns.org\""; //socketdial
const char CONNECT_response[] PROGMEM= "CONNECT";


const char ATSGACT_P_enable[] PROGMEM = "AT#SGACT=1,1"; //should report ipaddress
const char ATSGACT_P_disable[] PROGMEM = "AT#SGACT=1,0";

const char ATSGACT_P_query[] PROGMEM = "AT#SGACT?";
const char ATSGACT_P_query_response[] PROGMEM = "#SGACT: 1,1";

const char ATSH_P[] PROGMEM = "AT#SH=1";
const char ATSO_P[] PROGMEM = "AT#SO=1";

const char NO_CARRIER_P[] PROGMEM = "NO CARRIER";
const char ESCAPE_SOCKET_P[] PROGMEM = "+++";



void init_telit_pwr_switch(void){

	DDRG |= (1<<PG0);
	PORTG &= ~(1<<PG0);

}

//return 1 if successfully powered on
//can be used to check if telit already responsive to AT-commands
unsigned char telit_pwr_switch_on(void){

	char buf[100];
	char *found = NULL;
	USART_rx_flush();

	request_modem(AT_P, 500, FALSE, buf);
	found = strstr_P(buf, OK_P);

	if(found != NULL){

		return 1;
	}
	else{

		PORTG |= (1<<PG0);
		_DELAY_MS(1300);
		PORTG &= ~(1<<PG0);
		_DELAY_MS(1000);
		USART_rx_flush();

		request_modem(AT_P, 100, FALSE, buf);
		found = strstr_P(buf, OK_P);

		if(found != NULL){
			return 1;
		}
		else{
			return 0;
		}
	}

}

unsigned char telit_attention(void){

	char buf[100];
	char *found = NULL;
	request_modem(AT_P, 100, FALSE, buf);
	found = strstr_P(buf, OK_P);

	if(found != NULL){
		return 1;
	}
	else{
		return 0;
	}

}


//return 1 if successfully powered off
unsigned char telit_pwr_switch_off(void){

	char buf[100];
	char *found = NULL;
	USART_rx_flush();

	request_modem(AT_P, 200, FALSE, buf);
	found = strstr_P(buf, OK_P);

	if(found != NULL){
		PORTG |= (1<<PG0);
		_DELAY_MS(1300);
		PORTG &= ~(1<<PG0);
		_DELAY_MS(2000);
		USART_rx_flush();

		request_modem(AT_P, 200, FALSE, buf);
		found = strstr_P(buf, OK_P);

		if(found != NULL){

			return 0;
		}
		else{
			return 1;

		}

	}
	else{
		return 1;

	}


}

void telit_pwr_reset(void){

	telit_pwr_switch_off();

	_DELAY_MS(500);

	telit_pwr_switch_on();

}

/*
 * Sends a request to the modem and stores the response into
 * the buffer.
 *	returns 0 if no response
 */
unsigned char request_modem(const char *command, uint16_t timeout, unsigned char check, char *buf) {

	unsigned char count = 0;
	char *found = NULL;

	//printf_P(command);

	uart_puts_P(command);
	USART_rx_flush(); 	//remove echo characters
	uputchar('\r');

	count = uart_gets_timeout(buf, timeout);
	if (count) {
		if (check) {
			found = strstr_P(buf, OK_P);
			if (found) {
				printf_P(ok_P);
			}
			else {
				printf_P(got_P, buf);
			}
		}
	}
	else {
		return 0;
	}
	return count;
}



/*
 * Gets a response from the modem.
 * returns 0 if no response
 */
unsigned char uart_gets_timeout(char *buf, uint16_t timeout) {
	unsigned char count = 0;
	unsigned char us_count = 0;
	uint16_t c;
	*buf = 0;
	while (timeout) {
		c = ugetchar0();
		if (c == UART_NO_DATA) {
			_delay_us(10);

			if (us_count++ == 100) {
				timeout--;
				us_count = 0;

			}
		}
		else {
			count++;
			*buf++ = c;
		}
	}
	if (count != 0) {
		*buf = 0;
		count++;
	}
	return count;
}






//query modem and check if gprscontext active and connected
//returns 1 if connected, 0 otherwise
unsigned char gprs_is_connected(void){
	char *found = NULL;
	char buf[100];
	request_modem(ATSGACT_P_query, 200, FALSE, buf);

	found = strstr_P(buf, ATSGACT_P_query_response);

	if(found != NULL){
		return 1;

	}
	else{
		return 0;
	}
}

//connect to gprs
//return 1 if success, 0 otherwise
unsigned char gprs_connect(void){
	char buf[100];

	if(gprs_is_connected()){
		return 1;
	}
	else{

		request_modem(ATSGACT_P_enable, 1000, FALSE, buf);
		_DELAY_MS(2000);

		return gprs_is_connected();
	}
}

//disconnect gprs connection
//return 1 if success, 0 otherwise
unsigned char gprs_disconnect(void){
	unsigned char connected = 0;
	char buf[100];

	request_modem(ATSGACT_P_disable, 200, FALSE, buf);
	connected = gprs_is_connected();

	if(connected){
		return 0;

	}
	else{
		return 1;

	}
}



unsigned char open_socket(void){

	char buf[200];
	char *found = NULL;
	char *found_no_carrier = NULL;

	request_modem(ATSD_P, 3500, FALSE, buf);
	printf("open %s",buf);

	found = strstr_P(buf, CONNECT_response);
	found_no_carrier = strstr_P(buf, NO_CARRIER_P);

	if(found != NULL && found_no_carrier==NULL){
		return 1;

	}
	else{
		return 0;

	}

}


unsigned char resume_socket(void){
	//AT#SO=1
	char buf[100];
	char *found = NULL;

	request_modem(ATSO_P, 1000, FALSE, buf);
	printf("resume %s",buf);

	found = strstr_P(buf, CONNECT_response);

	if(found != NULL){
		return 1;

	}
	else{
		return 0;

	}

}

unsigned char suspend_socket(void){
	//+++

	_DELAY_MS(1100);

	uputs("+++");

	_DELAY_MS(1100);

	return telit_attention();


}


unsigned char close_socket(void){

	//AT#SH=1
	char buf[100];
	char *found = NULL;

	request_modem(ATSH_P, 1500, FALSE, buf);
	printf("close %s",buf);
	found = strstr_P(buf, OK_P);

	if(found != NULL){
		return 1;

	}
	else{
		return 0;

	}
}


unsigned char get_gps_string(char *buf){

	request_modem(ATGPSACP_P, 200, FALSE, buf);

	return 0;

}
